@@ -5,12 +5,16 @@
 #ifndef V8_OBJECTS_OBJECTS_BODY_DESCRIPTORS_INL_H_
 #define V8_OBJECTS_OBJECTS_BODY_DESCRIPTORS_INL_H_
 
+#include "src/objects/objects-body-descriptors.h"
+// Include the non-inl header before the rest of the headers.
+
 #include <algorithm>
 
 #include "include/v8-internal.h"
 #include "src/base/logging.h"
 #include "src/codegen/reloc-info.h"
 #include "src/common/globals.h"
+#include "src/heap/heap-layout-inl.h"
 #include "src/ic/handler-configuration.h"
 #include "src/objects/arguments-inl.h"
 #include "src/objects/bigint.h"
@@ -32,12 +36,12 @@
 #include "src/objects/js-weak-refs.h"
 #include "src/objects/literal-objects.h"
 #include "src/objects/megadom-handler-inl.h"
-#include "src/objects/objects-body-descriptors.h"
 #include "src/objects/ordered-hash-table-inl.h"
 #include "src/objects/property-descriptor-object.h"
 #include "src/objects/source-text-module.h"
 #include "src/objects/swiss-name-dictionary-inl.h"
 #include "src/objects/synthetic-module.h"
+#include "src/objects/tagged-field.h"
 #include "src/objects/template-objects-inl.h"
 #include "src/objects/torque-defined-classes-inl.h"
 #include "src/objects/transitions.h"
@@ -106,7 +110,7 @@ void BodyDescriptorBase::IterateJSObjectBodyWithoutEmbedderFieldsImpl(
     ObjectVisitor* v) {
   // This body iteration assumes that there's no embedder fields.
   DCHECK_IMPLIES(JSObject::MayHaveEmbedderFields(map),
-                 UncheckedCast<JSObject>(obj)->GetEmbedderFieldCount() == 0);
+                 UncheckedCast<JSObject>(obj)->GetEmbedderFieldCount(map) == 0);
   IteratePointers(obj, start_offset, end_offset, v);
 }
 
@@ -284,7 +288,8 @@ void BodyDescriptorBase::IterateProtectedPointer(Tagged<HeapObject> obj,
 template <typename ObjectVisitor>
 void BodyDescriptorBase::IterateJSDispatchEntry(Tagged<HeapObject> obj,
                                                 int offset, ObjectVisitor* v) {
-  JSDispatchHandle handle = obj->Relaxed_ReadField<JSDispatchHandle>(offset);
+  JSDispatchHandle handle(
+      obj->Relaxed_ReadField<JSDispatchHandle::underlying_type>(offset));
   v->VisitJSDispatchTableEntry(obj, handle);
 }
 #endif  // V8_ENABLE_LEAPTIERING
@@ -381,6 +386,12 @@ class RegExpData::BodyDescriptor final : public BodyDescriptorBase {
   template <typename ObjectVisitor>
   static inline void IterateBody(Tagged<Map> map, Tagged<HeapObject> obj,
                                  int object_size, ObjectVisitor* v) {
+    // If new pointers are added to RegExpData, make sure to also add them to
+    // the subclasses descriptors (AtomRegExpData and IrRegExpData).
+    // We don't directly call the base class IterateBody, as in the future
+    // the subclasses will have a different indirect pointer tag from the base
+    // class (once inheritance hierarchies are supported for indirect pointer
+    // tags).
     IterateSelfIndirectPointer(obj, kRegExpDataIndirectPointerTag, v);
     IteratePointer(obj, kSourceOffset, v);
     IteratePointer(obj, kWrapperOffset, v);
@@ -419,10 +430,10 @@ class IrRegExpData::BodyDescriptor final : public BodyDescriptorBase {
     IteratePointer(obj, kSourceOffset, v);
     IteratePointer(obj, kWrapperOffset, v);
 
-    IterateCodePointer(obj, kLatin1CodeOffset, v, IndirectPointerMode::kStrong);
-    IterateCodePointer(obj, kUc16CodeOffset, v, IndirectPointerMode::kStrong);
     IterateProtectedPointer(obj, kLatin1BytecodeOffset, v);
     IterateProtectedPointer(obj, kUc16BytecodeOffset, v);
+    IterateCodePointer(obj, kLatin1CodeOffset, v, IndirectPointerMode::kStrong);
+    IterateCodePointer(obj, kUc16CodeOffset, v, IndirectPointerMode::kStrong);
     IteratePointer(obj, kCaptureNameMapOffset, v);
   }
 
@@ -525,6 +536,11 @@ class AllocationSite::BodyDescriptor final : public BodyDescriptorBase {
 class JSFunction::BodyDescriptor final : public BodyDescriptorBase {
  public:
   static const int kStartOffset = JSObject::BodyDescriptor::kStartOffset;
+#ifdef V8_ENABLE_LEAPTIERING
+  static const int kCodeFieldOffset = JSFunction::kDispatchHandleOffset;
+#else
+  static const int kCodeFieldOffset = JSFunction::kCodeOffset;
+#endif
 
   template <typename ObjectVisitor>
   static inline void IterateBody(Tagged<Map> map, Tagged<HeapObject> obj,
@@ -532,8 +548,11 @@ class JSFunction::BodyDescriptor final : public BodyDescriptorBase {
     // Iterate JSFunction header fields first.
     int header_size = JSFunction::GetHeaderSize(map->has_prototype_slot());
     DCHECK_GE(object_size, header_size);
-    IteratePointers(obj, kStartOffset, kCodeOffset, v);
+    IteratePointers(obj, kStartOffset, kCodeFieldOffset, v);
 
+#ifdef V8_ENABLE_LEAPTIERING
+    IterateJSDispatchEntry(obj, kDispatchHandleOffset, v);
+#else
     // The code field is treated as a custom weak pointer. This field
     // is visited as a weak pointer if the Code is baseline code
     // and the bytecode array corresponding to this function is old. In the rest
@@ -541,13 +560,10 @@ class JSFunction::BodyDescriptor final : public BodyDescriptorBase {
     // See MarkingVisitorBase::VisitJSFunction.
     IterateCodePointer(obj, kCodeOffset, v, IndirectPointerMode::kCustom);
     DCHECK_GE(header_size, kCodeOffset);
-
-#ifdef V8_ENABLE_LEAPTIERING
-    IterateJSDispatchEntry(obj, kDispatchHandleOffset, v);
-#endif
+#endif  // V8_ENABLE_LEAPTIERING
 
     // Iterate rest of the header fields
-    IteratePointers(obj, kCodeOffset + kTaggedSize, header_size, v);
+    IteratePointers(obj, kCodeFieldOffset + kTaggedSize, header_size, v);
     // Iterate rest of the fields starting after the header.
     IterateJSObjectBodyImpl(map, obj, header_size, object_size, v);
   }
@@ -762,33 +778,6 @@ class BytecodeWrapper::BodyDescriptor final : public BodyDescriptorBase {
   }
 };
 
-class ExternalPointerArray::BodyDescriptor final : public BodyDescriptorBase {
- public:
-  template <typename ObjectVisitor>
-  static inline void IterateBody(Tagged<Map> map, Tagged<HeapObject> obj,
-                                 int object_size, ObjectVisitor* v) {
-    Tagged<ExternalPointerArray> array =
-        UncheckedCast<ExternalPointerArray>(obj);
-    for (int i = 0; i < array->length(); i++) {
-      // We don't currently track the (expected) tag of the elements of this
-      // array, so we have to use the generic tag here. This is ok as long as
-      // the visitor does not try to dereference the pointer (which it never
-      // should). The alternative would probably be to store the tag as
-      // additional metadata in the array itself, but then we also need to be
-      // careful since an attacker could then modify the tag.
-      v->VisitExternalPointer(array,
-                              array->RawExternalPointerField(
-                                  ExternalPointerArray::OffsetOfElementAt(i),
-                                  kAnyExternalPointerTag));
-    }
-  }
-
-  static inline int SizeOf(Tagged<Map> map, Tagged<HeapObject> obj) {
-    return ExternalPointerArray::SizeFor(
-        Cast<ExternalPointerArray>(obj)->length(kAcquireLoad));
-  }
-};
-
 class BigInt::BodyDescriptor final : public DataOnlyBodyDescriptor {
  public:
   static inline int SizeOf(Tagged<Map> map, Tagged<HeapObject> obj) {
@@ -1025,13 +1014,6 @@ class WasmTypeInfo::BodyDescriptor final : public BodyDescriptorBase {
   template <typename ObjectVisitor>
   static inline void IterateBody(Tagged<Map> map, Tagged<HeapObject> obj,
                                  int object_size, ObjectVisitor* v) {
-    v->VisitExternalPointer(
-        obj, obj->RawExternalPointerField(kNativeTypeOffset,
-                                          kWasmTypeInfoNativeTypeTag));
-
-    IterateTrustedPointer(obj, kTrustedDataOffset, v,
-                          IndirectPointerMode::kStrong,
-                          kWasmTrustedInstanceDataIndirectPointerTag);
     IteratePointers(obj, kSupertypesOffset, SizeOf(map, obj), v);
   }
 
@@ -1041,6 +1023,18 @@ class WasmTypeInfo::BodyDescriptor final : public BodyDescriptorBase {
   }
 };
 
+class WasmMemoryMapDescriptor::BodyDescriptor : public BodyDescriptorBase {
+ public:
+  static inline void IterateBody(Tagged<Map> map, Tagged<HeapObject> obj,
+                                 int object_size, ObjectVisitor* v) {
+    IterateMaybeWeakPointer(obj, kMemoryOffset, v);
+  }
+
+  static inline int SizeOf(Tagged<Map> map, Tagged<HeapObject> object) {
+    return map->instance_size();
+  }
+};
+
 class WasmInstanceObject::BodyDescriptor final : public BodyDescriptorBase {
  public:
   template <typename ObjectVisitor>
@@ -1088,12 +1082,14 @@ class WasmTableObject::BodyDescriptor final : public BodyDescriptorBase {
   static inline void IterateBody(Tagged<Map> map, Tagged<HeapObject> obj,
                                  int object_size, ObjectVisitor* v) {
     IteratePointers(obj, JSObject::BodyDescriptor::kStartOffset,
-                    kTrustedDataOffset, v);
+                    kTrustedDispatchTableOffset, v);
+    IterateTrustedPointer(obj, kTrustedDispatchTableOffset, v,
+                          IndirectPointerMode::kStrong,
+                          kWasmDispatchTableIndirectPointerTag);
     IterateTrustedPointer(obj, kTrustedDataOffset, v,
                           IndirectPointerMode::kStrong,
                           kWasmTrustedInstanceDataIndirectPointerTag);
-    IterateJSObjectBodyImpl(map, obj, kIsTable64Offset + kTaggedSize,
-                            object_size, v);
+    IterateJSObjectBodyImpl(map, obj, kHeaderSize, object_size, v);
   }
 
   static inline int SizeOf(Tagged<Map> map, Tagged<HeapObject> object) {
@@ -1145,6 +1141,9 @@ class WasmDispatchTable::BodyDescriptor final : public BodyDescriptorBase {
   template <typename ObjectVisitor>
   static inline void IterateBody(Tagged<Map> map, Tagged<HeapObject> obj,
                                  int object_size, ObjectVisitor* v) {
+    IterateSelfIndirectPointer(obj, kWasmDispatchTableIndirectPointerTag, v);
+    IterateProtectedPointer(obj, kProtectedOffheapDataOffset, v);
+    IterateProtectedPointer(obj, kProtectedUsesOffset, v);
     int length = Cast<WasmDispatchTable>(obj)->length(kAcquireLoad);
     for (int i = 0; i < length; ++i) {
       IterateProtectedPointer(obj, OffsetOf(i) + kImplicitArgBias, v);
@@ -1162,9 +1161,7 @@ class WasmArray::BodyDescriptor final : public BodyDescriptorBase {
   template <typename ObjectVisitor>
   static inline void IterateBody(Tagged<Map> map, Tagged<HeapObject> obj,
                                  int object_size, ObjectVisitor* v) {
-    // The type is safe to use because it's kept alive by the {map}'s
-    // WasmTypeInfo.
-    if (!WasmArray::GcSafeType(map)->element_type().is_reference()) return;
+    if (!WasmArray::GcSafeElementType(map).is_reference()) return;
     IteratePointers(obj, WasmArray::kHeaderSize, object_size, v);
   }
 
@@ -1179,9 +1176,12 @@ class WasmStruct::BodyDescriptor final : public BodyDescriptorBase {
   static inline void IterateBody(Tagged<Map> map, Tagged<HeapObject> obj,
                                  int object_size, ObjectVisitor* v) {
     Tagged<WasmStruct> wasm_struct = UncheckedCast<WasmStruct>(obj);
-    // The {type} is safe to use because it's kept alive by the {map}'s
-    // WasmTypeInfo.
-    wasm::StructType* type = WasmStruct::GcSafeType(map);
+    const wasm::CanonicalStructType* type = WasmStruct::GcSafeType(map);
+    if (type->is_descriptor()) {
+      // The associated Map is stored where the first field would otherwise be.
+      DCHECK_NE(type->field_offset(0), 0);
+      v->VisitPointer(wasm_struct, wasm_struct->RawField(0));
+    }
     for (uint32_t i = 0; i < type->field_count(); i++) {
       if (!type->field(i).is_reference()) continue;
       int offset = static_cast<int>(type->field_offset(i));
@@ -1206,6 +1206,20 @@ class WasmNull::BodyDescriptor : public DataOnlyBodyDescriptor {
   }
 };
 
+class WasmMemoryObject::BodyDescriptor final : public BodyDescriptorBase {
+ public:
+  template <typename ObjectVisitor>
+  static inline void IterateBody(Tagged<Map> map, Tagged<HeapObject> obj,
+                                 int object_size, ObjectVisitor* v) {
+    IteratePointers(obj, JSObject::BodyDescriptor::kStartOffset,
+                    kEndOfStrongFieldsOffset, v);
+    IterateJSObjectBodyImpl(map, obj, kHeaderSize, object_size, v);
+  }
+
+  static inline int SizeOf(Tagged<Map> map, Tagged<HeapObject> object) {
+    return map->instance_size();
+  }
+};
 #endif  // V8_ENABLE_WEBASSEMBLY
 
 class ExternalString::BodyDescriptor final : public BodyDescriptorBase {
@@ -1344,8 +1358,11 @@ class Code::BodyDescriptor final : public BodyDescriptorBase {
 
     static_assert(Code::kEndOfStrongFieldsWithMainCageBaseOffset ==
                   Code::kInstructionStreamOffset);
-    static_assert(Code::kInstructionStreamOffset + kTaggedSize ==
-                  Code::kEndOfStrongFieldsOffset);
+
+#ifdef V8_ENABLE_LEAPTIERING
+    IterateJSDispatchEntry(obj, kDispatchHandleOffset, v);
+#endif  // V8_ENABLE_LEAPTIERING
+
     v->VisitInstructionStreamPointer(
         Cast<Code>(obj),
         obj->RawInstructionStreamField(kInstructionStreamOffset));
@@ -1398,385 +1415,15 @@ class EmbedderDataArray::BodyDescriptor final : public BodyDescriptorBase {
   }
 };
 
-template <typename Op, typename... Args>
-auto BodyDescriptorApply(InstanceType type, Args&&... args) {
-#define CALL_APPLY(ClassName)                                         \
-  Op::template apply<ObjectTraits<ClassName>::BodyDescriptor, false>( \
-      std::forward<Args>(args)...)
-#define CALL_APPLY_TRUSTED(ClassName)                                \
-  Op::template apply<ObjectTraits<ClassName>::BodyDescriptor, true>( \
-      std::forward<Args>(args)...)
-
-  if (type < FIRST_NONSTRING_TYPE) {
-    switch (type & kStringRepresentationMask) {
-      case kSeqStringTag:
-        if ((type & kStringEncodingMask) == kOneByteStringTag) {
-          return CALL_APPLY(SeqOneByteString);
-        } else {
-          return CALL_APPLY(SeqTwoByteString);
-        }
-      case kConsStringTag:
-        return CALL_APPLY(ConsString);
-      case kThinStringTag:
-        return CALL_APPLY(ThinString);
-      case kSlicedStringTag:
-        return CALL_APPLY(SlicedString);
-      case kExternalStringTag:
-        return CALL_APPLY(ExternalString);
-    }
-    UNREACHABLE();
-  }
-  if (InstanceTypeChecker::IsJSApiObject(type)) {
-    return CALL_APPLY(JSAPIObjectWithEmbedderSlots);
-  }
-
-  switch (type) {
-    case EMBEDDER_DATA_ARRAY_TYPE:
-      return CALL_APPLY(EmbedderDataArray);
-    case HASH_TABLE_TYPE:
-    case ORDERED_HASH_MAP_TYPE:
-    case ORDERED_HASH_SET_TYPE:
-    case ORDERED_NAME_DICTIONARY_TYPE:
-    case NAME_DICTIONARY_TYPE:
-    case GLOBAL_DICTIONARY_TYPE:
-    case NUMBER_DICTIONARY_TYPE:
-    case SIMPLE_NUMBER_DICTIONARY_TYPE:
-    case NAME_TO_INDEX_HASH_TABLE_TYPE:
-    case REGISTERED_SYMBOL_TABLE_TYPE:
-      return CALL_APPLY(FixedArray);
-#define CASE(TypeCamelCase, TYPE_UPPER_CASE) \
-  case TYPE_UPPER_CASE##_TYPE:               \
-    return CALL_APPLY(TypeCamelCase);
-      SIMPLE_HEAP_OBJECT_LIST2(CASE)
-#undef CASE
-    case SLOPPY_ARGUMENTS_ELEMENTS_TYPE:
-      return CALL_APPLY(SloppyArgumentsElements);
-    case EPHEMERON_HASH_TABLE_TYPE:
-      return CALL_APPLY(EphemeronHashTable);
-    case AWAIT_CONTEXT_TYPE:
-    case BLOCK_CONTEXT_TYPE:
-    case CATCH_CONTEXT_TYPE:
-    case DEBUG_EVALUATE_CONTEXT_TYPE:
-    case EVAL_CONTEXT_TYPE:
-    case FUNCTION_CONTEXT_TYPE:
-    case MODULE_CONTEXT_TYPE:
-    case SCRIPT_CONTEXT_TYPE:
-    case WITH_CONTEXT_TYPE:
-      return CALL_APPLY(Context);
-    case NATIVE_CONTEXT_TYPE:
-      return CALL_APPLY(NativeContext);
-    case FEEDBACK_METADATA_TYPE:
-      return CALL_APPLY(FeedbackMetadata);
-    case PROPERTY_ARRAY_TYPE:
-      return CALL_APPLY(PropertyArray);
-    case TRANSITION_ARRAY_TYPE:
-      return CALL_APPLY(TransitionArray);
-    case FEEDBACK_CELL_TYPE:
-      return CALL_APPLY(FeedbackCell);
-    case COVERAGE_INFO_TYPE:
-      return CALL_APPLY(CoverageInfo);
-#if V8_ENABLE_WEBASSEMBLY
-    case WASM_ARRAY_TYPE:
-      return CALL_APPLY(WasmArray);
-    case WASM_EXCEPTION_PACKAGE_TYPE:
-      return CALL_APPLY(WasmExceptionPackage);
-    case WASM_FUNC_REF_TYPE:
-      return CALL_APPLY(WasmFuncRef);
-    case WASM_GLOBAL_OBJECT_TYPE:
-      return CALL_APPLY(WasmGlobalObject);
-    case WASM_RESUME_DATA_TYPE:
-      return CALL_APPLY(WasmResumeData);
-    case WASM_CONTINUATION_OBJECT_TYPE:
-      return CALL_APPLY(WasmContinuationObject);
-    case WASM_STRUCT_TYPE:
-      return CALL_APPLY(WasmStruct);
-    case WASM_TABLE_OBJECT_TYPE:
-      return CALL_APPLY(WasmTableObject);
-    case WASM_TYPE_INFO_TYPE:
-      return CALL_APPLY(WasmTypeInfo);
-    case WASM_SUSPENDER_OBJECT_TYPE:
-      return CALL_APPLY(WasmSuspenderObject);
-    case WASM_SUSPENDING_OBJECT_TYPE:
-      return CALL_APPLY(WasmSuspendingObject);
-#endif  // V8_ENABLE_WEBASSEMBLY
-    case JS_ARGUMENTS_OBJECT_TYPE:
-    case JS_ARRAY_ITERATOR_PROTOTYPE_TYPE:
-    case JS_ARRAY_ITERATOR_TYPE:
-    case JS_ARRAY_TYPE:
-    case JS_ASYNC_DISPOSABLE_STACK_TYPE:
-    case JS_ASYNC_FROM_SYNC_ITERATOR_TYPE:
-    case JS_ASYNC_FUNCTION_OBJECT_TYPE:
-    case JS_ASYNC_GENERATOR_OBJECT_TYPE:
-    case JS_BOUND_FUNCTION_TYPE:
-    case JS_CONTEXT_EXTENSION_OBJECT_TYPE:
-    case JS_DISPOSABLE_STACK_BASE_TYPE:
-    case JS_ERROR_TYPE:
-    case JS_FINALIZATION_REGISTRY_TYPE:
-    case JS_GENERATOR_OBJECT_TYPE:
-    case JS_ITERATOR_FILTER_HELPER_TYPE:
-    case JS_ITERATOR_MAP_HELPER_TYPE:
-    case JS_ITERATOR_TAKE_HELPER_TYPE:
-    case JS_ITERATOR_DROP_HELPER_TYPE:
-    case JS_ITERATOR_FLAT_MAP_HELPER_TYPE:
-    case JS_ITERATOR_PROTOTYPE_TYPE:
-    case JS_MAP_ITERATOR_PROTOTYPE_TYPE:
-    case JS_MAP_KEY_ITERATOR_TYPE:
-    case JS_MAP_KEY_VALUE_ITERATOR_TYPE:
-    case JS_MAP_TYPE:
-    case JS_MAP_VALUE_ITERATOR_TYPE:
-    case JS_MESSAGE_OBJECT_TYPE:
-    case JS_MODULE_NAMESPACE_TYPE:
-    case JS_OBJECT_PROTOTYPE_TYPE:
-    case JS_OBJECT_TYPE:
-    case JS_PRIMITIVE_WRAPPER_TYPE:
-    case JS_PROMISE_PROTOTYPE_TYPE:
-    case JS_PROMISE_TYPE:
-    case JS_REG_EXP_PROTOTYPE_TYPE:
-    case JS_REG_EXP_STRING_ITERATOR_TYPE:
-    case JS_SET_ITERATOR_PROTOTYPE_TYPE:
-    case JS_SET_KEY_VALUE_ITERATOR_TYPE:
-    case JS_SET_PROTOTYPE_TYPE:
-    case JS_SET_TYPE:
-    case JS_SET_VALUE_ITERATOR_TYPE:
-    case JS_SHADOW_REALM_TYPE:
-    case JS_SHARED_ARRAY_TYPE:
-    case JS_SHARED_STRUCT_TYPE:
-    case JS_STRING_ITERATOR_PROTOTYPE_TYPE:
-    case JS_STRING_ITERATOR_TYPE:
-    case JS_SYNC_DISPOSABLE_STACK_TYPE:
-    case JS_TEMPORAL_CALENDAR_TYPE:
-    case JS_TEMPORAL_DURATION_TYPE:
-    case JS_TEMPORAL_INSTANT_TYPE:
-    case JS_TEMPORAL_PLAIN_DATE_TYPE:
-    case JS_TEMPORAL_PLAIN_DATE_TIME_TYPE:
-    case JS_TEMPORAL_PLAIN_MONTH_DAY_TYPE:
-    case JS_TEMPORAL_PLAIN_TIME_TYPE:
-    case JS_TEMPORAL_PLAIN_YEAR_MONTH_TYPE:
-    case JS_TEMPORAL_TIME_ZONE_TYPE:
-    case JS_TEMPORAL_ZONED_DATE_TIME_TYPE:
-    case JS_TYPED_ARRAY_PROTOTYPE_TYPE:
-    case JS_VALID_ITERATOR_WRAPPER_TYPE:
-    case JS_WRAPPED_FUNCTION_TYPE:
-    case JS_RAW_JSON_TYPE:
-#ifdef V8_INTL_SUPPORT
-    case JS_V8_BREAK_ITERATOR_TYPE:
-    case JS_COLLATOR_TYPE:
-    case JS_DATE_TIME_FORMAT_TYPE:
-    case JS_DISPLAY_NAMES_TYPE:
-    case JS_DURATION_FORMAT_TYPE:
-    case JS_LIST_FORMAT_TYPE:
-    case JS_LOCALE_TYPE:
-    case JS_NUMBER_FORMAT_TYPE:
-    case JS_PLURAL_RULES_TYPE:
-    case JS_RELATIVE_TIME_FORMAT_TYPE:
-    case JS_SEGMENT_ITERATOR_TYPE:
-    case JS_SEGMENTER_TYPE:
-    case JS_SEGMENTS_TYPE:
-#endif  // V8_INTL_SUPPORT
-#if V8_ENABLE_WEBASSEMBLY
-    case WASM_MEMORY_OBJECT_TYPE:
-    case WASM_MODULE_OBJECT_TYPE:
-    case WASM_TAG_OBJECT_TYPE:
-    case WASM_VALUE_OBJECT_TYPE:
-#endif  // V8_ENABLE_WEBASSEMBLY
-      return CALL_APPLY(JSObject);
-
-    case JS_DATE_TYPE:
-      return CALL_APPLY(JSDate);
-
-    case JS_API_OBJECT_TYPE:
-    case JS_GLOBAL_OBJECT_TYPE:
-    case JS_GLOBAL_PROXY_TYPE:
-    case JS_SPECIAL_API_OBJECT_TYPE:
-      return Op::template apply<JSAPIObjectWithEmbedderSlots::BodyDescriptor,
-                                false>(std::forward<Args>(args)...);
-
-    case JS_FUNCTION_TYPE:
-    case JS_CLASS_CONSTRUCTOR_TYPE:
-    case JS_PROMISE_CONSTRUCTOR_TYPE:
-    case JS_REG_EXP_CONSTRUCTOR_TYPE:
-    case JS_ARRAY_CONSTRUCTOR_TYPE:
-#define TYPED_ARRAY_CONSTRUCTORS_SWITCH(Type, type, TYPE, Ctype) \
-  case TYPE##_TYPED_ARRAY_CONSTRUCTOR_TYPE:
-      TYPED_ARRAYS(TYPED_ARRAY_CONSTRUCTORS_SWITCH)
-#undef TYPED_ARRAY_CONSTRUCTORS_SWITCH
-      return CALL_APPLY(JSFunction);
-#if V8_ENABLE_WEBASSEMBLY
-    case WASM_INSTANCE_OBJECT_TYPE:
-      return CALL_APPLY(WasmInstanceObject);
-    case WASM_NULL_TYPE:
-      return CALL_APPLY(WasmNull);
-#endif  // V8_ENABLE_WEBASSEMBLY
-    case JS_WEAK_MAP_TYPE:
-    case JS_WEAK_SET_TYPE:
-      return CALL_APPLY(JSWeakCollection);
-    case JS_ARRAY_BUFFER_TYPE:
-      return CALL_APPLY(JSArrayBuffer);
-    case JS_DATA_VIEW_TYPE:
-      return CALL_APPLY(JSDataView);
-    case JS_RAB_GSAB_DATA_VIEW_TYPE:
-      return CALL_APPLY(JSRabGsabDataView);
-    case JS_TYPED_ARRAY_TYPE:
-      return CALL_APPLY(JSTypedArray);
-    case JS_EXTERNAL_OBJECT_TYPE:
-      return CALL_APPLY(JSExternalObject);
-    case WEAK_CELL_TYPE:
-      return CALL_APPLY(WeakCell);
-    case JS_WEAK_REF_TYPE:
-      return CALL_APPLY(JSWeakRef);
-    case JS_PROXY_TYPE:
-      return CALL_APPLY(JSProxy);
-    case JS_ATOMICS_MUTEX_TYPE:
-    case JS_ATOMICS_CONDITION_TYPE:
-      return CALL_APPLY(JSSynchronizationPrimitive);
-    case FOREIGN_TYPE:
-      return CALL_APPLY(Foreign);
-    case MAP_TYPE:
-      return CALL_APPLY(Map);
-    case CELL_TYPE:
-      return CALL_APPLY(Cell);
-    case PROPERTY_CELL_TYPE:
-      return CALL_APPLY(PropertyCell);
-    case CONST_TRACKING_LET_CELL_TYPE:
-      return CALL_APPLY(ConstTrackingLetCell);
-    case SYMBOL_TYPE:
-      return CALL_APPLY(Symbol);
-    case SMALL_ORDERED_HASH_SET_TYPE:
-      return CALL_APPLY(SmallOrderedHashTable<SmallOrderedHashSet>);
-    case SMALL_ORDERED_HASH_MAP_TYPE:
-      return CALL_APPLY(SmallOrderedHashTable<SmallOrderedHashMap>);
-    case SMALL_ORDERED_NAME_DICTIONARY_TYPE:
-      return CALL_APPLY(SmallOrderedHashTable<SmallOrderedNameDictionary>);
-    case SWISS_NAME_DICTIONARY_TYPE:
-      return CALL_APPLY(SwissNameDictionary);
-    case PREPARSE_DATA_TYPE:
-      return CALL_APPLY(PreparseData);
-    case SHARED_FUNCTION_INFO_TYPE:
-      return CALL_APPLY(SharedFunctionInfo);
-    case HEAP_NUMBER_TYPE:
-      return CALL_APPLY(HeapNumber);
-    case EXTERNAL_POINTER_ARRAY_TYPE:
-      return CALL_APPLY(ExternalPointerArray);
-    case BIGINT_TYPE:
-      return CALL_APPLY(BigInt);
-    case ALLOCATION_SITE_TYPE:
-      return CALL_APPLY(AllocationSite);
-    case ODDBALL_TYPE:
-      return CALL_APPLY(Oddball);
-    case HOLE_TYPE:
-      return CALL_APPLY(Hole);
-    case JS_REG_EXP_TYPE:
-      return CALL_APPLY(JSRegExp);
-
-#define MAKE_STRUCT_CASE(TYPE, Name, name) \
-  case TYPE:                               \
-    return CALL_APPLY(Name);
-      STRUCT_LIST(MAKE_STRUCT_CASE)
-#undef MAKE_STRUCT_CASE
-    case ACCESSOR_INFO_TYPE:
-      return CALL_APPLY(AccessorInfo);
-    case FUNCTION_TEMPLATE_INFO_TYPE:
-      return CALL_APPLY(FunctionTemplateInfo);
-    case OBJECT_TEMPLATE_INFO_TYPE:
-      return CALL_APPLY(ObjectTemplateInfo);
-    case LOAD_HANDLER_TYPE:
-      return CALL_APPLY(LoadHandler);
-    case STORE_HANDLER_TYPE:
-      return CALL_APPLY(StoreHandler);
-    case SOURCE_TEXT_MODULE_TYPE:
-      return CALL_APPLY(SourceTextModule);
-    case SYNTHETIC_MODULE_TYPE:
-      return CALL_APPLY(SyntheticModule);
-// TODO(turbofan): Avoid duplicated cases when the body descriptors are
-// identical.
-#define MAKE_TORQUE_BODY_DESCRIPTOR_APPLY(TYPE, TypeName) \
-  case TYPE:                                              \
-    return CALL_APPLY(TypeName);
-      TORQUE_INSTANCE_TYPE_TO_BODY_DESCRIPTOR_LIST(
-          MAKE_TORQUE_BODY_DESCRIPTOR_APPLY)
-#undef MAKE_TORQUE_BODY_DESCRIPTOR_APPLY
-
-#define MAKE_TRUSTED_OBJECT_CASE(TypeCamelCase, TYPE_UPPER_CASE) \
-  case TYPE_UPPER_CASE##_TYPE:                                   \
-    return CALL_APPLY_TRUSTED(TypeCamelCase);
-      CONCRETE_TRUSTED_OBJECT_TYPE_LIST2(MAKE_TRUSTED_OBJECT_CASE)
-#undef MAKE_TRUSTED_OBJECT_CASE
-
-    case FILLER_TYPE:
-      return Op::template apply<FreeSpaceFillerBodyDescriptor, false>(
-          std::forward<Args>(args)...);
-
-    case FREE_SPACE_TYPE:
-      return CALL_APPLY(FreeSpace);
-
-    default:
-      PrintF("Unknown type: %d\n", type);
-      UNREACHABLE();
-  }
-#undef CALL_APPLY
-}
-
-template <typename ObjectVisitor>
-void HeapObject::IterateFast(PtrComprCageBase cage_base, ObjectVisitor* v) {
-  v->VisitMapPointer(*this);
-  IterateBodyFast(cage_base, v);
-}
-
-template <typename ObjectVisitor>
-void HeapObject::IterateFast(Tagged<Map> map, ObjectVisitor* v) {
-  v->VisitMapPointer(*this);
-  IterateBodyFast(map, SizeFromMap(map), v);
-}
-
-template <typename ObjectVisitor>
-void HeapObject::IterateFast(Tagged<Map> map, int object_size,
-                             ObjectVisitor* v) {
-  v->VisitMapPointer(*this);
-  IterateBodyFast(map, object_size, v);
-}
-
-template <typename ObjectVisitor>
-void HeapObject::IterateBodyFast(PtrComprCageBase cage_base, ObjectVisitor* v) {
-  Tagged<Map> m = map(cage_base);
-  IterateBodyFast(m, SizeFromMap(m), v);
-}
-
-struct CallIterateBody {
-  template <typename BodyDescriptor, bool kIsTrusted, typename ObjectVisitor>
-  static void apply(Tagged<Map> map, Tagged<HeapObject> obj, int object_size,
-                    ObjectVisitor* v) {
-    if constexpr (kIsTrusted) {
-      DCHECK(IsTrustedObject(obj));
-      // This CHECK defends against an attacker crafting fake trusted objects
-      // inside the sandbox. See also crbug.com/c/1505089 for more details.
-      SBXCHECK(OutsideSandboxOrInReadonlySpace(obj));
-    } else {
-      // If this DCHECK fails, it means that a trusted object was not added to
-      // the HEAP_OBJECT_TRUSTED_TYPE_LIST.
-      // Note: This would normally be just !IsTrustedObject(obj), however we
-      // might see trusted objects here before they've been migrated to trusted
-      // space, hence the second condition.
-      DCHECK(!IsTrustedObject(obj) || !IsTrustedSpaceObject(obj));
-    }
-    BodyDescriptor::IterateBody(map, obj, object_size, v);
-  }
-};
-
-template <typename ObjectVisitor>
-void HeapObject::IterateBodyFast(Tagged<Map> map, int object_size,
-                                 ObjectVisitor* v) {
-  BodyDescriptorApply<CallIterateBody>(map->instance_type(), map, *this,
-                                       object_size, v);
-}
-
 class EphemeronHashTable::BodyDescriptor final : public BodyDescriptorBase {
  public:
   template <typename ObjectVisitor>
   static inline void IterateBody(Tagged<Map> map, Tagged<HeapObject> obj,
                                  int object_size, ObjectVisitor* v) {
-    int entries_start = EphemeronHashTable::kHeaderSize +
-                        EphemeronHashTable::kElementsStartIndex * kTaggedSize;
-    IteratePointers(obj, EphemeronHashTable::kHeaderSize, entries_start, v);
+    int entries_start = EphemeronHashTable::OffsetOfElementAt(
+        EphemeronHashTable::kElementsStartIndex);
+    IteratePointers(obj, OFFSET_OF_DATA_START(EphemeronHashTable),
+                    entries_start, v);
     Tagged<EphemeronHashTable> table = UncheckedCast<EphemeronHashTable>(obj);
     for (InternalIndex i : table->IterateEntries()) {
       const int key_index = EphemeronHashTable::EntryToIndex(i);
@@ -1858,8 +1505,8 @@ class ProtectedFixedArray::BodyDescriptor final : public BodyDescriptorBase {
   template <typename ObjectVisitor>
   static inline void IterateBody(Tagged<Map> map, Tagged<HeapObject> obj,
                                  int object_size, ObjectVisitor* v) {
-    for (int offset = kHeaderSize; offset < object_size;
-         offset += kTaggedSize) {
+    for (int offset = OFFSET_OF_DATA_START(ProtectedFixedArray);
+         offset < object_size; offset += kTaggedSize) {
       IterateProtectedPointer(obj, offset, v);
     }
   }
@@ -1951,6 +1598,25 @@ class TrustedWeakFixedArray::BodyDescriptor final
   }
 };
 
+class ProtectedWeakFixedArray::BodyDescriptor final
+    : public BodyDescriptorBase {
+ public:
+  template <typename ObjectVisitor>
+  static inline void IterateBody(Tagged<Map> map, Tagged<HeapObject> obj,
+                                 int object_size, ObjectVisitor* v) {
+    Tagged<TrustedObject> host = Cast<TrustedObject>(obj);
+    for (int offset = OFFSET_OF_DATA_START(ProtectedWeakFixedArray);
+         offset < object_size; offset += kTaggedSize) {
+      v->VisitProtectedPointer(host,
+                               host->RawProtectedMaybeObjectField(offset));
+    }
+  }
+
+  static inline int SizeOf(Tagged<Map> map, Tagged<HeapObject> raw_object) {
+    return UncheckedCast<ProtectedWeakFixedArray>(raw_object)->AllocatedSize();
+  }
+};
+
 #include "torque-generated/objects-body-descriptors-inl.inc"
 
 }  // namespace internal
