@@ -47,6 +47,7 @@ const {
   HTTPParser,
   isLenient,
   prepareError,
+  kSkipPendingData,
 } = require('_http_common');
 const {
   kUniqueHeaders,
@@ -64,6 +65,8 @@ const {
   traceBegin,
   traceEnd,
   getNextTraceEventId,
+  kProxyConfig,
+  checkShouldUseProxy,
 } = require('internal/http');
 const {
   ConnResetException,
@@ -78,6 +81,7 @@ const {
 const {
   validateInteger,
   validateBoolean,
+  validateString,
 } = require('internal/validators');
 const { getTimerDuration } = require('internal/timers');
 const {
@@ -89,6 +93,7 @@ const {
 const kClientRequestStatistics = Symbol('ClientRequestStatistics');
 
 const dc = require('diagnostics_channel');
+const onClientRequestCreatedChannel = dc.channel('http.client.request.created');
 const onClientRequestStartChannel = dc.channel('http.client.request.start');
 const onClientRequestErrorChannel = dc.channel('http.client.request.error');
 const onClientResponseFinishChannel = dc.channel('http.client.response.finish');
@@ -133,6 +138,56 @@ class HTTPClientAsyncResource {
   }
 }
 
+// When proxying a HTTP request, the following needs to be done:
+// https://datatracker.ietf.org/doc/html/rfc7230#section-5.3.2
+// 1. Rewrite the request path to absolute-form.
+// 2. Add proxy-connection and proxy-authorization headers appropriately.
+//
+// This function checks whether the request should be rewritten for proxying
+// and modifies the headers as well as req.path if necessary.
+// The handling of the proxy server connection is done in createConnection.
+function rewriteForProxiedHttp(req, reqOptions) {
+  if (req._header) {
+    debug('request._header is already sent, skipping rewriteForProxiedHttp', reqOptions);
+    return false;
+  }
+  const agent = req.agent;
+  if (!agent || !agent[kProxyConfig]) {
+    return false;
+  }
+  if ((reqOptions.protocol || agent.protocol) !== 'http:') {
+    return false;
+  }
+  // TODO(joyeecheung): cache this on the req or on the agent.
+  const shouldUseProxy = checkShouldUseProxy(agent[kProxyConfig], reqOptions);
+  debug(`rewriteForProxiedHttp should use proxy for ${reqOptions.host}:${reqOptions.port}:`, shouldUseProxy);
+  if (!shouldUseProxy) {
+    return false;
+  }
+  // Add proxy headers.
+  const { auth, href } = agent[kProxyConfig];
+  if (auth) {
+    req.setHeader('proxy-authorization', auth);
+  }
+  if (req.shouldKeepAlive) {
+    req.setHeader('proxy-connection', 'keep-alive');
+  } else {
+    req.setHeader('proxy-connection', 'close');
+  }
+
+  // Convert the path to absolute-form.
+  // https://datatracker.ietf.org/doc/html/rfc7230#section-5.3.2
+  const requestHost = req.getHeader('host') || 'localhost';
+  const requestBase = `http://${requestHost}`;
+  const requestURL = new URL(req.path, requestBase);
+  if (reqOptions.port) {
+    requestURL.port = reqOptions.port;
+  }
+  req.path = requestURL.href;
+  debug(`updated request for HTTP proxy ${href} with ${req.path} `, req[kOutHeaders]);
+  return true;
+};
+
 function ClientRequest(input, options, cb) {
   OutgoingMessage.call(this);
 
@@ -198,7 +253,13 @@ function ClientRequest(input, options, cb) {
   const host = optsWithoutSignal.host = validateHost(options.hostname, 'hostname') ||
                                         validateHost(options.host, 'host') || 'localhost';
 
-  const setHost = (options.setHost === undefined || Boolean(options.setHost));
+  const setHost = options.setHost !== undefined ?
+    Boolean(options.setHost) :
+    options.setDefaultHeaders !== false;
+
+  this._removedConnection = options.setDefaultHeaders === false;
+  this._removedContLen = options.setDefaultHeaders === false;
+  this._removedTE = options.setDefaultHeaders === false;
 
   this.socketPath = options.socketPath;
 
@@ -211,12 +272,11 @@ function ClientRequest(input, options, cb) {
     delete optsWithoutSignal.signal;
   }
   let method = options.method;
-  const methodIsString = (typeof method === 'string');
-  if (method !== null && method !== undefined && !methodIsString) {
-    throw new ERR_INVALID_ARG_TYPE('options.method', 'string', method);
+  if (method != null) {
+    validateString(method, 'options.method');
   }
 
-  if (methodIsString && method) {
+  if (method) {
     if (!checkIsHttpToken(method)) {
       throw new ERR_INVALID_HTTP_TOKEN('Method', method);
     }
@@ -325,10 +385,14 @@ function ClientRequest(input, options, cb) {
         throw new ERR_HTTP_HEADERS_SENT('render');
       }
 
+      rewriteForProxiedHttp(this, optsWithoutSignal);
       this._storeHeader(this.method + ' ' + this.path + ' HTTP/1.1\r\n',
                         this[kOutHeaders]);
+    } else {
+      rewriteForProxiedHttp(this, optsWithoutSignal);
     }
   } else {
+    rewriteForProxiedHttp(this, optsWithoutSignal);
     this._storeHeader(this.method + ' ' + this.path + ' HTTP/1.1\r\n',
                       options.headers);
   }
@@ -373,6 +437,11 @@ function ClientRequest(input, options, cb) {
       this.onSocket(net.createConnection(opts));
     }
   }
+  if (onClientRequestCreatedChannel.hasSubscribers) {
+    onClientRequestCreatedChannel.publish({
+      request: this,
+    });
+  }
 }
 ObjectSetPrototypeOf(ClientRequest.prototype, OutgoingMessage.prototype);
 ObjectSetPrototypeOf(ClientRequest, OutgoingMessage);
@@ -624,7 +693,14 @@ function parserOnIncomingClient(res, shouldKeepAlive) {
     // We already have a response object, this means the server
     // sent a double response.
     socket.destroy();
-    return 0;  // No special treatment.
+    if (socket.parser) {
+      // https://github.com/nodejs/node/issues/60025
+      // Now, parser.incoming is pointed to the new IncomingMessage,
+      // we need to rewrite it to the first one and skip all the pending IncomingMessage
+      socket.parser.incoming = req.res;
+      socket.parser.incoming[kSkipPendingData] = true;
+    }
+    return 0;
   }
   req.res = res;
 
@@ -882,7 +958,8 @@ function onSocketNT(req, socket, err) {
       if (!req.aborted && !err) {
         err = new ConnResetException('socket hang up');
       }
-      if (err) {
+      // ERR_PROXY_TUNNEL is handled by the proxying logic
+      if (err && err.code !== 'ERR_PROXY_TUNNEL') {
         emitErrorEvent(req, err);
       }
       req._closed = true;
