@@ -74,6 +74,7 @@ const {
   defaultTriggerAsyncIdScope,
   symbols: { async_id_symbol, owner_symbol },
 } = require('internal/async_hooks');
+const { FastBuffer } = require('internal/buffer');
 const { UV_UDP_REUSEADDR } = internalBinding('constants').os;
 
 const {
@@ -550,6 +551,8 @@ function fixBufferList(list) {
     const buf = list[i];
     if (typeof buf === 'string')
       newlist[i] = Buffer.from(buf);
+    else if (Buffer.isBuffer(buf))
+      newlist[i] = buf;
     else if (!isArrayBufferView(buf))
       return null;
     else
@@ -687,7 +690,7 @@ Socket.prototype.send = function(buffer,
     this.bind({ port: 0, exclusive: true }, null);
 
   if (list.length === 0)
-    ArrayPrototypePush(list, Buffer.alloc(0));
+    ArrayPrototypePush(list, new FastBuffer());
 
   // If the socket hasn't been bound yet, push the outbound packet onto the
   // send queue and send after binding is complete.
