@@ -28,6 +28,7 @@ const {
   ArrayPrototypeSlice,
   ArrayPrototypeSplice,
   ArrayPrototypeUnshift,
+  AsyncIteratorPrototype,
   Boolean,
   Error,
   ErrorCaptureStackTrace,
@@ -157,8 +158,8 @@ function lazyEventEmitterAsyncResource() {
         }
 
         /**
-         * @param {symbol,string} event
-         * @param  {...any} args
+         * @param {symbol|string} event
+         * @param {any[]} args
          * @returns {boolean}
          */
         emit(event, ...args) {
@@ -203,7 +204,7 @@ function lazyEventEmitterAsyncResource() {
 /**
  * Creates a new `EventEmitter` instance.
  * @param {{ captureRejections?: boolean; }} [opts]
- * @constructs {EventEmitter}
+ * @constructs EventEmitter
  */
 function EventEmitter(opts) {
   EventEmitter.init.call(this, opts);
@@ -276,12 +277,6 @@ ObjectDefineProperty(EventEmitter, 'defaultMaxListeners', {
   },
 });
 
-function hasEventListener(self, type) {
-  if (type === undefined)
-    return self._events !== undefined;
-  return self._events !== undefined && self._events[type] !== undefined;
-};
-
 ObjectDefineProperties(EventEmitter, {
   kMaxEventTargetListeners: {
     __proto__: null,
@@ -675,11 +670,13 @@ EventEmitter.prototype.removeListener =
     function removeListener(type, listener) {
       checkListener(listener);
 
-      if (!hasEventListener(this, type))
+      const events = this._events;
+      if (events === undefined)
         return this;
 
-      const events = this._events;
       const list = events[type];
+      if (list === undefined)
+        return this;
 
       if (list === listener || list.listener === listener) {
         this._eventsCount -= 1;
@@ -690,9 +687,10 @@ EventEmitter.prototype.removeListener =
           this._events = { __proto__: null };
         } else {
           delete events[type];
-          if (events.removeListener)
-            this.emit('removeListener', type, list.listener || listener);
         }
+
+        if (events.removeListener !== undefined)
+          this.emit('removeListener', type, list.listener || listener);
       } else if (typeof list !== 'function') {
         let position = -1;
 
@@ -733,9 +731,9 @@ EventEmitter.prototype.off = EventEmitter.prototype.removeListener;
  */
 EventEmitter.prototype.removeAllListeners =
     function removeAllListeners(type) {
-      if (!hasEventListener(this))
-        return this;
       const events = this._events;
+      if (events === undefined)
+        return this;
 
       // Not listening for removeListener, no need to emit
       if (events.removeListener === undefined) {
@@ -780,10 +778,14 @@ EventEmitter.prototype.removeAllListeners =
     };
 
 function _listeners(target, type, unwrap) {
-  if (!hasEventListener(target, type))
+  const events = target._events;
+
+  if (events === undefined)
     return [];
 
-  const evlistener = target._events[type];
+  const evlistener = events[type];
+  if (evlistener === undefined)
+    return [];
 
   if (typeof evlistener === 'function')
     return unwrap ? [evlistener.listener || evlistener] : [evlistener];
@@ -871,7 +873,7 @@ function listenerCount(type, listener) {
 /**
  * Returns an array listing the events for which
  * the emitter has registered listeners.
- * @returns {any[]}
+ * @returns {(string | symbol)[]}
  */
 EventEmitter.prototype.eventNames = function eventNames() {
   return this._eventsCount > 0 ? ReflectOwnKeys(this._events) : [];
@@ -939,7 +941,7 @@ function getEventListeners(emitterOrTarget, type) {
 function getMaxListeners(emitterOrTarget) {
   if (typeof emitterOrTarget?.getMaxListeners === 'function') {
     return _getMaxListeners(emitterOrTarget);
-  } else if (emitterOrTarget?.[kMaxEventTargetListeners]) {
+  } else if (typeof emitterOrTarget?.[kMaxEventTargetListeners] === 'number') {
     return emitterOrTarget[kMaxEventTargetListeners];
   }
 
@@ -1000,9 +1002,6 @@ async function once(emitter, name, options = kEmptyObject) {
   });
 }
 
-const AsyncIteratorPrototype = ObjectGetPrototypeOf(
-  ObjectGetPrototypeOf(async function* () {}).prototype);
-
 function createIterResult(value, done) {
   return { value, done };
 }
@@ -1115,24 +1114,28 @@ function on(emitter, event, options = kEmptyObject) {
     [kWatermarkData]: {
       /**
        * The current queue size
+       * @returns {number}
        */
       get size() {
         return size;
       },
       /**
        * The low watermark. The emitter is resumed every time size is lower than it
+       * @returns {number}
        */
       get low() {
         return lowWatermark;
       },
       /**
        * The high watermark. The emitter is paused every time size is higher than it
+       * @returns {number}
        */
       get high() {
         return highWatermark;
       },
       /**
        * It checks whether the emitter is paused by the watermark controller or not
+       * @returns {boolean}
        */
       get isPaused() {
         return paused;
