@@ -34,6 +34,7 @@ const {
   ObjectDefineProperties,
   ObjectDefineProperty,
   Promise,
+  PromisePrototypeThen,
   PromiseResolve,
   ReflectApply,
   SafeMap,
@@ -41,6 +42,7 @@ const {
   StringPrototypeCharCodeAt,
   StringPrototypeIndexOf,
   StringPrototypeSlice,
+  SymbolDispose,
   uncurryThis,
 } = primordials;
 
@@ -60,7 +62,6 @@ const {
 } = constants;
 
 const pathModule = require('path');
-const { isAbsolute } = pathModule;
 const { isArrayBufferView } = require('internal/util/types');
 
 const binding = internalBinding('fs');
@@ -87,7 +88,7 @@ const {
 const { toPathIfFileURL } = require('internal/url');
 const {
   customPromisifyArgs: kCustomPromisifyArgsSymbol,
-  emitExperimentalWarning,
+  deprecate,
   getLazy,
   kEmptyObject,
   promisify: {
@@ -152,7 +153,6 @@ const {
 
 const permission = require('internal/process/permission');
 
-let truncateWarn = true;
 let fs;
 
 // Lazy loaded
@@ -169,15 +169,10 @@ let ReadFileContext;
 // monkeypatching.
 let FileReadStream;
 let FileWriteStream;
+let Utf8Stream;
 
-function showTruncateDeprecation() {
-  if (truncateWarn) {
-    process.emitWarning(
-      'Using fs.truncate with a file descriptor is deprecated. Please use ' +
-      'fs.ftruncate with a file descriptor instead.',
-      'DeprecationWarning', 'DEP0081');
-    truncateWarn = false;
-  }
+function lazyLoadUtf8Stream() {
+  Utf8Stream ??= require('internal/streams/fast-utf8-stream');
 }
 
 // Ensure that callbacks run in the global context. Only use this function
@@ -272,12 +267,7 @@ ObjectDefineProperty(exists, kCustomPromisifiedSymbol, {
   },
 });
 
-// fs.existsSync never throws, it only returns true or false.
-// Since fs.existsSync never throws, users have established
-// the expectation that passing invalid arguments to it, even like
-// fs.existsSync(), would only get a false in return, so we cannot signal
-// validation errors to users properly out of compatibility concerns.
-// TODO(joyeecheung): deprecate the never-throw-on-invalid-arguments behavior
+let showExistsDeprecation = true;
 /**
  * Synchronously tests whether or not the given path exists.
  * @param {string | Buffer | URL} path
@@ -286,7 +276,13 @@ ObjectDefineProperty(exists, kCustomPromisifiedSymbol, {
 function existsSync(path) {
   try {
     path = getValidatedPath(path);
-  } catch {
+  } catch (err) {
+    if (showExistsDeprecation && err?.code === 'ERR_INVALID_ARG_TYPE') {
+      process.emitWarning(
+        'Passing invalid argument types to fs.existsSync is deprecated', 'DeprecationWarning', 'DEP0187',
+      );
+      showExistsDeprecation = false;
+    }
     return false;
   }
 
@@ -605,7 +601,6 @@ function openAsBlob(path, options = kEmptyObject) {
  */
 function read(fd, buffer, offsetOrOptions, length, position, callback) {
   fd = getValidatedFd(fd);
-
   let offset = offsetOrOptions;
   let params = null;
   if (arguments.length <= 4) {
@@ -697,8 +692,6 @@ ObjectDefineProperty(read, kCustomPromisifyArgsSymbol,
  * @returns {number}
  */
 function readSync(fd, buffer, offsetOrOptions, length, position) {
-  fd = getValidatedFd(fd);
-
   validateBuffer(buffer);
 
   let offset = offsetOrOptions;
@@ -787,7 +780,6 @@ ObjectDefineProperty(readv, kCustomPromisifyArgsSymbol,
  * @returns {number}
  */
 function readvSync(fd, buffers, position) {
-  fd = getValidatedFd(fd);
   validateBufferArray(buffers);
 
   if (typeof position !== 'number')
@@ -817,7 +809,6 @@ function write(fd, buffer, offsetOrOptions, length, position, callback) {
   }
 
   fd = getValidatedFd(fd);
-
   let offset = offsetOrOptions;
   if (isArrayBufferView(buffer)) {
     callback ||= position || length || offset;
@@ -888,7 +879,6 @@ ObjectDefineProperty(write, kCustomPromisifyArgsSymbol,
  * @returns {number}
  */
 function writeSync(fd, buffer, offsetOrOptions, length, position) {
-  fd = getValidatedFd(fd);
   const ctx = {};
   let result;
 
@@ -978,7 +968,6 @@ ObjectDefineProperty(writev, kCustomPromisifyArgsSymbol, {
  * @returns {number}
  */
 function writevSync(fd, buffers, position) {
-  fd = getValidatedFd(fd);
   validateBufferArray(buffers);
 
   if (buffers.length === 0) {
@@ -1033,10 +1022,6 @@ function renameSync(oldPath, newPath) {
  * @returns {void}
  */
 function truncate(path, len, callback) {
-  if (typeof path === 'number') {
-    showTruncateDeprecation();
-    return fs.ftruncate(path, len, callback);
-  }
   if (typeof len === 'function') {
     callback = len;
     len = 0;
@@ -1066,11 +1051,6 @@ function truncate(path, len, callback) {
  * @returns {void}
  */
 function truncateSync(path, len) {
-  if (typeof path === 'number') {
-    // legacy
-    showTruncateDeprecation();
-    return fs.ftruncateSync(path, len);
-  }
   if (len === undefined) {
     len = 0;
   }
@@ -1131,11 +1111,7 @@ function lazyLoadRimraf() {
 /**
  * Asynchronously removes a directory.
  * @param {string | Buffer | URL} path
- * @param {{
- *   maxRetries?: number;
- *   recursive?: boolean;
- *   retryDelay?: number;
- *   }} [options]
+ * @param {object} [options]
  * @param {(err?: Error) => any} callback
  * @returns {void}
  */
@@ -1179,11 +1155,7 @@ function rmdir(path, options, callback) {
 /**
  * Synchronously removes a directory.
  * @param {string | Buffer | URL} path
- * @param {{
- *   maxRetries?: number;
- *   recursive?: boolean;
- *   retryDelay?: number;
- *   }} [options]
+ * @param {object} [options]
  * @returns {void}
  */
 function rmdirSync(path, options) {
@@ -1259,6 +1231,11 @@ function rmSync(path, options) {
 function fdatasync(fd, callback) {
   const req = new FSReqCallback();
   req.oncomplete = makeCallback(callback);
+
+  if (permission.isEnabled()) {
+    callback(new ERR_ACCESS_DENIED('fdatasync API is disabled when Permission Model is enabled.'));
+    return;
+  }
   binding.fdatasync(fd, req);
 }
 
@@ -1270,6 +1247,9 @@ function fdatasync(fd, callback) {
  * @returns {void}
  */
 function fdatasyncSync(fd) {
+  if (permission.isEnabled()) {
+    throw new ERR_ACCESS_DENIED('fdatasync API is disabled when Permission Model is enabled.');
+  }
   binding.fdatasync(fd);
 }
 
@@ -1283,6 +1263,10 @@ function fdatasyncSync(fd) {
 function fsync(fd, callback) {
   const req = new FSReqCallback();
   req.oncomplete = makeCallback(callback);
+  if (permission.isEnabled()) {
+    callback(new ERR_ACCESS_DENIED('fsync API is disabled when Permission Model is enabled.'));
+    return;
+  }
   binding.fsync(fd, req);
 }
 
@@ -1293,6 +1277,9 @@ function fsync(fd, callback) {
  * @returns {void}
  */
 function fsyncSync(fd) {
+  if (permission.isEnabled()) {
+    throw new ERR_ACCESS_DENIED('fsync API is disabled when Permission Model is enabled.');
+  }
   binding.fsync(fd);
 }
 
@@ -1446,7 +1433,7 @@ function handleDirents({ result, currentPath, context }) {
     const dirent = getDirent(currentPath, names[i], types[i]);
     ArrayPrototypePush(context.readdirResults, dirent);
 
-    if (dirent.isDirectory() || binding.internalModuleStat(binding, fullPath) === 1) {
+    if (dirent.isDirectory() || binding.internalModuleStat(fullPath) === 1) {
       ArrayPrototypePush(context.pathsQueue, fullPath);
     }
   }
@@ -1456,7 +1443,7 @@ function handleFilePaths({ result, currentPath, context }) {
   for (let i = 0; i < result.length; i++) {
     const resultPath = pathModule.join(currentPath, result[i]);
     const relativeResultPath = pathModule.relative(context.basePath, resultPath);
-    const stat = binding.internalModuleStat(binding, resultPath);
+    const stat = binding.internalModuleStat(resultPath);
     ArrayPrototypePush(context.readdirResults, relativeResultPath);
 
     if (stat === 1) {
@@ -1676,7 +1663,7 @@ function statfs(path, options = { bigint: false }, callback) {
 
     callback(err, getStatFsFromBinding(stats));
   };
-  binding.statfs(getValidatedPath(path), options.bigint, req);
+  binding.statfs(path, options.bigint, req);
 }
 
 /**
@@ -1711,7 +1698,7 @@ function lstatSync(path, options = { bigint: false, throwIfNoEntry: true }) {
     throw new ERR_ACCESS_DENIED('Access to this API has been restricted', 'FileSystemRead', resource);
   }
   const stats = binding.lstat(
-    getValidatedPath(path),
+    path,
     options.bigint,
     undefined,
     options.throwIfNoEntry,
@@ -1798,18 +1785,12 @@ function symlink(target, path, type, callback) {
     validateOneOf(type, 'type', ['dir', 'file', 'junction', null, undefined]);
   }
 
-  if (permission.isEnabled()) {
-    // The permission model's security guarantees fall apart in the presence of
-    // relative symbolic links. Thus, we have to prevent their creation.
-    if (BufferIsBuffer(target)) {
-      if (!isAbsolute(BufferToString(target))) {
-        callback(new ERR_ACCESS_DENIED('relative symbolic link target'));
-        return;
-      }
-    } else if (typeof target !== 'string' || !isAbsolute(toPathIfFileURL(target))) {
-      callback(new ERR_ACCESS_DENIED('relative symbolic link target'));
-      return;
-    }
+  // Due to the nature of Node.js runtime, symlinks has different edge cases that can bypass
+  // the permission model security guarantees. Thus, this API is disabled unless fs.read
+  // and fs.write permission has been given.
+  if (permission.isEnabled() && !permission.has('fs')) {
+    callback(new ERR_ACCESS_DENIED('fs.symlink API requires full fs.read and fs.write permissions.'));
+    return;
   }
 
   target = getValidatedPath(target, 'target');
@@ -1873,16 +1854,11 @@ function symlinkSync(target, path, type) {
     }
   }
 
-  if (permission.isEnabled()) {
-    // The permission model's security guarantees fall apart in the presence of
-    // relative symbolic links. Thus, we have to prevent their creation.
-    if (BufferIsBuffer(target)) {
-      if (!isAbsolute(BufferToString(target))) {
-        throw new ERR_ACCESS_DENIED('relative symbolic link target');
-      }
-    } else if (typeof target !== 'string' || !isAbsolute(toPathIfFileURL(target))) {
-      throw new ERR_ACCESS_DENIED('relative symbolic link target');
-    }
+  // Due to the nature of Node.js runtime, symlinks has different edge cases that can bypass
+  // the permission model security guarantees. Thus, this API is disabled unless fs.read
+  // and fs.write permission has been given.
+  if (permission.isEnabled() && !permission.has('fs')) {
+    throw new ERR_ACCESS_DENIED('fs.symlink API requires full fs.read and fs.write permissions.');
   }
 
   target = getValidatedPath(target, 'target');
@@ -2223,6 +2199,11 @@ function futimes(fd, atime, mtime, callback) {
   mtime = toUnixTimestamp(mtime, 'mtime');
   callback = makeCallback(callback);
 
+  if (permission.isEnabled()) {
+    callback(new ERR_ACCESS_DENIED('futimes API is disabled when Permission Model is enabled.'));
+    return;
+  }
+
   const req = new FSReqCallback();
   req.oncomplete = callback;
   binding.futimes(fd, atime, mtime, req);
@@ -2238,6 +2219,10 @@ function futimes(fd, atime, mtime, callback) {
  * @returns {void}
  */
 function futimesSync(fd, atime, mtime) {
+  if (permission.isEnabled()) {
+    throw new ERR_ACCESS_DENIED('futimes API is disabled when Permission Model is enabled.');
+  }
+
   binding.futimes(
     fd,
     toUnixTimestamp(atime, 'atime'),
@@ -3051,6 +3036,36 @@ function mkdtempSync(prefix, options) {
   return binding.mkdtemp(prefix, options.encoding);
 }
 
+/**
+ * Synchronously creates a unique temporary directory.
+ * The returned value is a disposable object which removes the
+ * directory and its contents when disposed.
+ * @param {string | Buffer | URL} prefix
+ * @param {string | { encoding?: string; }} [options]
+ * @returns {object} A disposable object with a "path" property.
+ */
+function mkdtempDisposableSync(prefix, options) {
+  options = getOptions(options);
+
+  prefix = getValidatedPath(prefix, 'prefix');
+  warnOnNonPortableTemplate(prefix);
+
+  const path = binding.mkdtemp(prefix, options.encoding);
+  // Stash the full path in case of process.chdir()
+  const fullPath = pathModule.resolve(process.cwd(), path);
+
+  const remove = () => {
+    binding.rmSync(fullPath, 0 /* maxRetries */, true /* recursive */, 100 /* retryDelay */);
+  };
+  return {
+    path,
+    remove,
+    [SymbolDispose]() {
+      remove();
+    },
+  };
+}
+
 /**
  * Asynchronously copies `src` to `dest`. By
  * default, `dest` is overwritten if it already exists.
@@ -3187,7 +3202,6 @@ function createWriteStream(path, options) {
 const lazyGlob = getLazy(() => require('internal/fs/glob').Glob);
 
 function glob(pattern, options, callback) {
-  emitExperimentalWarning('glob');
   if (typeof options === 'function') {
     callback = options;
     options = undefined;
@@ -3195,19 +3209,14 @@ function glob(pattern, options, callback) {
   callback = makeCallback(callback);
 
   const Glob = lazyGlob();
-  // TODO: Use iterator helpers when available
-  (async () => {
-    try {
-      const res = await ArrayFromAsync(new Glob(pattern, options).glob());
-      callback(null, res);
-    } catch (err) {
-      callback(err);
-    }
-  })();
+  PromisePrototypeThen(
+    ArrayFromAsync(new Glob(pattern, options).glob()),
+    (res) => callback(null, res),
+    callback,
+  );
 }
 
 function globSync(pattern, options) {
-  emitExperimentalWarning('globSync');
   const Glob = lazyGlob();
   return new Glob(pattern, options).globSync();
 }
@@ -3262,6 +3271,7 @@ module.exports = fs = {
   mkdirSync,
   mkdtemp,
   mkdtempSync,
+  mkdtempDisposableSync,
   open,
   openSync,
   openAsBlob,
@@ -3345,6 +3355,11 @@ module.exports = fs = {
     FileWriteStream = val;
   },
 
+  get Utf8Stream() {
+    lazyLoadUtf8Stream();
+    return Utf8Stream;
+  },
+
   // For tests
   _toUnixTimestamp: toUnixTimestamp,
 };
@@ -3356,10 +3371,50 @@ defineLazyProperties(
 );
 
 ObjectDefineProperties(fs, {
-  F_OK: { __proto__: null, enumerable: true, value: F_OK || 0 },
-  R_OK: { __proto__: null, enumerable: true, value: R_OK || 0 },
-  W_OK: { __proto__: null, enumerable: true, value: W_OK || 0 },
-  X_OK: { __proto__: null, enumerable: true, value: X_OK || 0 },
+  F_OK: {
+    __proto__: null,
+    enumerable: false,
+    get: deprecate(
+      function get() {
+        return F_OK || 0;
+      },
+      'fs.F_OK is deprecated, use fs.constants.F_OK instead',
+      'DEP0176',
+    ),
+  },
+  R_OK: {
+    __proto__: null,
+    enumerable: false,
+    get: deprecate(
+      function get() {
+        return R_OK || 0;
+      },
+      'fs.R_OK is deprecated, use fs.constants.R_OK instead',
+      'DEP0176',
+    ),
+  },
+  W_OK: {
+    __proto__: null,
+    enumerable: false,
+    get: deprecate(
+      function get() {
+        return W_OK || 0;
+      },
+      'fs.W_OK is deprecated, use fs.constants.W_OK instead',
+      'DEP0176',
+    ),
+  },
+  X_OK: {
+    __proto__: null,
+    enumerable: false,
+    get: deprecate(
+      function get() {
+        return X_OK || 0;
+      },
+      'fs.X_OK is deprecated, use fs.constants.X_OK instead',
+      'DEP0176',
+    ),
+  },
   constants: {
     __proto__: null,
     configurable: false,
