@@ -2,8 +2,9 @@
 
 const {
   Array,
+  ArrayPrototypeFind,
   ArrayPrototypeJoin,
-  ArrayPrototypeSome,
+  ArrayPrototypePush,
   FunctionPrototype,
   ObjectSetPrototypeOf,
   PromisePrototypeThen,
@@ -36,7 +37,10 @@ const {
     entry_point_module_private_symbol,
   },
 } = internalBinding('util');
-const { decorateErrorStack, kEmptyObject } = require('internal/util');
+/**
+ * @typedef {import('./utils.js').ModuleRequestType} ModuleRequestType
+ */
+const { decorateErrorStack } = require('internal/util');
 const { isPromise } = require('internal/util/types');
 const {
   getSourceMapsSupport,
@@ -61,8 +65,8 @@ const CJSGlobalLike = [
   '__filename',
   '__dirname',
 ];
-const isCommonJSGlobalLikeNotDefinedError = (errorMessage) =>
-  ArrayPrototypeSome(
+const findCommonJSGlobalLikeNotDefinedError = (errorMessage) =>
+  ArrayPrototypeFind(
     CJSGlobalLike,
     (globalLike) => errorMessage === `${globalLike} is not defined`,
   );
@@ -75,11 +79,28 @@ const isCommonJSGlobalLikeNotDefinedError = (errorMessage) =>
  * @returns {void}
  */
 const explainCommonJSGlobalLikeNotDefinedError = (e, url, hasTopLevelAwait) => {
-  if (e?.name === 'ReferenceError' &&
-      isCommonJSGlobalLikeNotDefinedError(e.message)) {
+  const notDefinedGlobalLike = e?.name === 'ReferenceError' && findCommonJSGlobalLikeNotDefinedError(e.message);
 
+  if (notDefinedGlobalLike) {
     if (hasTopLevelAwait) {
-      e.message = `Cannot determine intended module format because both require() and top-level await are present. If the code is intended to be CommonJS, wrap await in an async function. If the code is intended to be an ES module, replace require() with import.`;
+      let advice;
+      switch (notDefinedGlobalLike) {
+        case 'require':
+          advice = 'replace require() with import';
+          break;
+        case 'module':
+        case 'exports':
+          advice = 'use export instead of module.exports/exports';
+          break;
+        case '__filename':
+          advice = 'use import.meta.filename instead';
+          break;
+        case '__dirname':
+          advice = 'use import.meta.dirname instead';
+          break;
+      }
+
+      e.message = `Cannot determine intended module format because both '${notDefinedGlobalLike}' and top-level await are present. If the code is intended to be CommonJS, wrap await in an async function. If the code is intended to be an ES module, ${advice}.`;
       e.code = 'ERR_AMBIGUOUS_MODULE_SYNTAX';
       return;
     }
@@ -106,8 +127,9 @@ const explainCommonJSGlobalLikeNotDefinedError = (e, url, hasTopLevelAwait) => {
 };
 
 class ModuleJobBase {
-  constructor(url, importAttributes, phase, isMain, inspectBrk) {
+  constructor(loader, url, importAttributes, phase, isMain, inspectBrk) {
     assert(typeof phase === 'number');
+    this.loader = loader;
     this.importAttributes = importAttributes;
     this.phase = phase;
     this.isMain = isMain;
@@ -115,13 +137,65 @@ class ModuleJobBase {
 
     this.url = url;
   }
+
+  /**
+   * Synchronously link the module and its dependencies.
+   * @param {ModuleRequestType} requestType Type of the module request.
+   * @returns {ModuleJobBase[]}
+   */
+  syncLink(requestType) {
+    // Store itself into the cache first before linking in case there are circular
+    // references in the linking.
+    this.loader.loadCache.set(this.url, this.type, this);
+    const moduleRequests = this.module.getModuleRequests();
+    // Modules should be aligned with the moduleRequests array in order.
+    const modules = Array(moduleRequests.length);
+    const evaluationDepJobs = [];
+    this.commonJsDeps = Array(moduleRequests.length);
+    try {
+      for (let idx = 0; idx < moduleRequests.length; idx++) {
+        const request = moduleRequests[idx];
+        // TODO(joyeecheung): split this into two iterators, one for resolving and one for loading so
+        // that hooks can pre-fetch sources off-thread.
+        const job = this.loader.getOrCreateModuleJob(this.url, request, requestType);
+        debug(`ModuleJobBase.syncLink() ${this.url} -> ${request.specifier}`, job);
+        assert(!isPromise(job));
+        assert(job.module instanceof ModuleWrap);
+        if (request.phase === kEvaluationPhase) {
+          ArrayPrototypePush(evaluationDepJobs, job);
+        }
+        modules[idx] = job.module;
+        this.commonJsDeps[idx] = job.module.isCommonJS;
+      }
+      this.module.link(modules);
+    } finally {
+      // Restore it - if it succeeds, we'll reset in the caller; Otherwise it's
+      // not cached and if the error is caught, subsequent attempt would still fail.
+      this.loader.loadCache.delete(this.url, this.type);
+    }
+
+    return evaluationDepJobs;
+  }
+
+  /**
+   * Ensure that this ModuleJob is moving towards the required phase
+   * (does not necessarily mean it is ready at that phase - run does that)
+   * @param {number} phase
+   */
+  ensurePhase(phase, requestType) {
+    if (this.phase < phase) {
+      this.phase = phase;
+      this.linked = this.link(requestType);
+      if (isPromise(this.linked)) {
+        PromisePrototypeThen(this.linked, undefined, noop);
+      }
+    }
+  }
 }
 
 /* A ModuleJob tracks the loading of a single Module, and the ModuleJobs of
  * its dependencies, over time. */
 class ModuleJob extends ModuleJobBase {
-  #loader = null;
-
   /**
    * @param {ModuleLoader} loader The ESM loader.
    * @param {string} url URL of the module to be wrapped in ModuleJob.
@@ -131,12 +205,11 @@ class ModuleJob extends ModuleJobBase {
    * @param {boolean} isMain Whether the module is the entry point.
    * @param {boolean} inspectBrk Whether this module should be evaluated with the
    *   first line paused in the debugger (because --inspect-brk is passed).
-   * @param {boolean} isForRequireInImportedCJS Whether this is created for require() in imported CJS.
+   * @param {ModuleRequestType} requestType Type of the module request.
    */
   constructor(loader, url, importAttributes = { __proto__: null }, moduleOrModulePromise,
-              phase = kEvaluationPhase, isMain, inspectBrk, isForRequireInImportedCJS = false) {
-    super(url, importAttributes, phase, isMain, inspectBrk);
-    this.#loader = loader;
+              phase = kEvaluationPhase, isMain, inspectBrk, requestType) {
+    super(loader, url, importAttributes, phase, isMain, inspectBrk);
 
     // Expose the promise to the ModuleWrap directly for linking below.
     if (isPromise(moduleOrModulePromise)) {
@@ -148,10 +221,12 @@ class ModuleJob extends ModuleJobBase {
 
     if (this.phase === kEvaluationPhase) {
       // Promise for the list of all dependencyJobs.
-      this.linked = this.#link();
+      this.linked = this.link(requestType);
       // This promise is awaited later anyway, so silence
       // 'unhandled rejection' warnings.
-      PromisePrototypeThen(this.linked, undefined, noop);
+      if (isPromise(this.linked)) {
+        PromisePrototypeThen(this.linked, undefined, noop);
+      }
     }
 
     // instantiated == deep dependency jobs wrappers are instantiated,
@@ -160,79 +235,69 @@ class ModuleJob extends ModuleJobBase {
   }
 
   /**
-   * Ensure that this ModuleJob is moving towards the required phase
-   * (does not necessarily mean it is ready at that phase - run does that)
-   * @param {number} phase
+   * @param {ModuleRequestType} requestType Type of the module request.
+   * @returns {ModuleJobBase[]|Promise<ModuleJobBase[]>}
    */
-  ensurePhase(phase) {
-    if (this.phase < phase) {
-      this.phase = phase;
-      this.linked = this.#link();
-      PromisePrototypeThen(this.linked, undefined, noop);
+  link(requestType) {
+    if (this.loader.isForAsyncLoaderHookWorker) {
+      return this.#asyncLink(requestType);
     }
+    return this.syncLink(requestType);
   }
 
   /**
-   * Iterates the module requests and links with the loader.
-   * @returns {Promise<ModuleJob[]>} Dependency module jobs.
+   * @param {ModuleRequestType} requestType Type of the module request.
+   * @returns {Promise<ModuleJobBase[]>}
    */
-  async #link() {
+  async #asyncLink(requestType) {
+    assert(this.loader.isForAsyncLoaderHookWorker);
     this.module = await this.modulePromise;
     assert(this.module instanceof ModuleWrap);
-
     const moduleRequests = this.module.getModuleRequests();
-    // Explicitly keeping track of dependency jobs is needed in order
-    // to flatten out the dependency graph below in `_instantiate()`,
-    // so that circular dependencies can't cause a deadlock by two of
-    // these `link` callbacks depending on each other.
     // Create an ArrayLike to avoid calling into userspace with `.then`
     // when returned from the async function.
-    const evaluationDepJobs = Array(moduleRequests.length);
-    ObjectSetPrototypeOf(evaluationDepJobs, null);
-
     // Modules should be aligned with the moduleRequests array in order.
     const modulePromises = Array(moduleRequests.length);
-    // Track each loop for whether it is an evaluation phase or source phase request.
-    let isEvaluation;
-    // Iterate with index to avoid calling into userspace with `Symbol.iterator`.
-    for (
-      let idx = 0, eidx = 0;
-      // Use the let-scoped eidx value to update the executionDepJobs length at the end of the loop.
-      idx < moduleRequests.length || (evaluationDepJobs.length = eidx, false);
-      idx++, eidx += isEvaluation
-    ) {
-      const { specifier, phase, attributes } = moduleRequests[idx];
-      isEvaluation = phase === kEvaluationPhase;
-      // TODO(joyeecheung): resolve all requests first, then load them in another
-      // loop so that hooks can pre-fetch sources off-thread.
-      const dependencyJobPromise = this.#loader.getModuleJobForImport(
-        specifier, this.url, attributes, phase,
-      );
+    const evaluationDepJobs = [];
+    this.commonJsDeps = Array(moduleRequests.length);
+    for (let idx = 0; idx < moduleRequests.length; idx++) {
+      const request = moduleRequests[idx];
+      // Explicitly keeping track of dependency jobs is needed in order
+      // to flatten out the dependency graph below in `asyncInstantiate()`,
+      // so that circular dependencies can't cause a deadlock by two of
+      // these `link` callbacks depending on each other.
+      // TODO(joyeecheung): split this into two iterators, one for resolving and one for loading so
+      // that hooks can pre-fetch sources off-thread.
+      const dependencyJobPromise = this.loader.getOrCreateModuleJob(this.url, request, requestType);
       const modulePromise = PromisePrototypeThen(dependencyJobPromise, (job) => {
-        debug(`async link() ${this.url} -> ${specifier}`, job);
-        if (phase === kEvaluationPhase) {
-          evaluationDepJobs[eidx] = job;
+        debug(`ModuleJob.asyncLink() ${this.url} -> ${request.specifier}`, job);
+        if (request.phase === kEvaluationPhase) {
+          ArrayPrototypePush(evaluationDepJobs, job);
         }
         return job.modulePromise;
       });
       modulePromises[idx] = modulePromise;
     }
-
     const modules = await SafePromiseAllReturnArrayLike(modulePromises);
-    this.module.link(modules);
+    for (let idx = 0; idx < moduleRequests.length; idx++) {
+      this.commonJsDeps[idx] = modules[idx].isCommonJS;
+    }
 
+    this.module.link(modules);
     return evaluationDepJobs;
   }
 
   #instantiate() {
     if (this.instantiated === undefined) {
-      this.instantiated = this.#_instantiate();
+      this.instantiated = this.#asyncInstantiate();
     }
     return this.instantiated;
   }
 
-  async #_instantiate() {
+  async #asyncInstantiate() {
     const jobsInGraph = new SafeSet();
+    // TODO(joyeecheung): if it's not on the async loader thread, consider this already
+    // linked.
     const addJobsToDependencyGraph = async (moduleJob) => {
       debug(`async addJobsToDependencyGraph() ${this.url}`, moduleJob);
 
@@ -240,7 +305,7 @@ class ModuleJob extends ModuleJobBase {
         return;
       }
       jobsInGraph.add(moduleJob);
-      const dependencyJobs = await moduleJob.linked;
+      const dependencyJobs = isPromise(moduleJob.linked) ? await moduleJob.linked : moduleJob.linked;
       return SafePromiseAllReturnVoid(dependencyJobs, addJobsToDependencyGraph);
     };
     await addJobsToDependencyGraph(this);
@@ -263,31 +328,19 @@ class ModuleJob extends ModuleJobBase {
           StringPrototypeIncludes(e.message,
                                   ' does not provide an export named')) {
         const splitStack = StringPrototypeSplit(e.stack, '\n', 2);
-        const parentFileUrl = RegExpPrototypeSymbolReplace(
-          /:\d+$/,
-          splitStack[0],
-          '',
-        );
         const { 1: childSpecifier, 2: name } = RegExpPrototypeExec(
           /module '(.*)' does not provide an export named '(.+)'/,
           e.message);
-        const { url: childFileURL } = await this.#loader.resolve(
-          childSpecifier,
-          parentFileUrl,
-          kEmptyObject,
-        );
-        let format;
-        try {
-          // This might throw for non-CommonJS modules because we aren't passing
-          // in the import attributes and some formats require them; but we only
-          // care about CommonJS for the purposes of this error message.
-          ({ format } =
-            await this.#loader.load(childFileURL));
-        } catch {
-          // Continue regardless of error.
+        const moduleRequests = this.module.getModuleRequests();
+        let isCommonJS = false;
+        for (let i = 0; i < moduleRequests.length; ++i) {
+          if (moduleRequests[i].specifier === childSpecifier) {
+            isCommonJS = this.commonJsDeps[i];
+            break;
+          }
         }
 
-        if (format === 'commonjs') {
+        if (isCommonJS) {
           const importStatement = splitStack[1];
           // TODO(@ctavan): The original error stack only provides the single
           // line which causes the error. For multi-line import statements we
@@ -324,17 +377,18 @@ class ModuleJob extends ModuleJobBase {
     let status = this.module.getStatus();
 
     debug('ModuleJob.runSync()', status, this.module);
-    // FIXME(joyeecheung): this cannot fully handle < kInstantiated. Make the linking
-    // fully synchronous instead.
     if (status === kUninstantiated) {
-      this.module.hasAsyncGraph = this.module.instantiateSync();
+      // FIXME(joyeecheung): this cannot fully handle < kInstantiated. Make the linking
+      // fully synchronous instead.
+      if (this.module.getModuleRequests().length === 0) {
+        this.module.link([]);
+      }
+      this.module.instantiate();
       status = this.module.getStatus();
     }
     if (status === kInstantiated || status === kErrored) {
       const filename = urlToFilename(this.url);
       const parentFilename = urlToFilename(parent?.filename);
-      this.module.hasAsyncGraph ??= this.module.isGraphAsync();
-
       if (this.module.hasAsyncGraph && !getOptionValue('--experimental-print-required-tla')) {
         throw new ERR_REQUIRE_ASYNC_MODULE(filename, parentFilename);
       }
@@ -345,6 +399,11 @@ class ModuleJob extends ModuleJobBase {
       }
       throw this.module.getError();
     } else if (status === kEvaluating || status === kEvaluated) {
+      if (this.module.hasAsyncGraph) {
+        const filename = urlToFilename(this.url);
+        const parentFilename = urlToFilename(parent?.filename);
+        throw new ERR_REQUIRE_ASYNC_MODULE(filename, parentFilename);
+      }
       // kEvaluating can show up when this is being used to deal with CJS <-> CJS cycles.
       // Allow it for now, since we only need to ban ESM <-> CJS cycles which would be
       // detected earlier during the linking phase, though the CJS handling in the ESM
@@ -352,7 +411,7 @@ class ModuleJob extends ModuleJobBase {
       // the CJS loader does.
       // TODO(joyeecheung): remove the re-invented require() in the ESM loader and
       // always handle CJS using the CJS loader to eliminate the quirks.
-      return { __proto__: null, module: this.module, namespace: this.module.getNamespaceSync() };
+      return { __proto__: null, module: this.module, namespace: this.module.getNamespace() };
     }
     assert.fail(`Unexpected module status ${status}.`);
   }
@@ -388,8 +447,6 @@ class ModuleJob extends ModuleJobBase {
  * TODO(joyeecheung): consolidate this with the isForRequireInImportedCJS variant of ModuleJob.
  */
 class ModuleJobSync extends ModuleJobBase {
-  #loader = null;
-
   /**
    * @param {ModuleLoader} loader The ESM loader.
    * @param {string} url URL of the module to be wrapped in ModuleJob.
@@ -401,57 +458,26 @@ class ModuleJobSync extends ModuleJobBase {
    *   first line paused in the debugger (because --inspect-brk is passed).
    */
   constructor(loader, url, importAttributes, moduleWrap, phase = kEvaluationPhase, isMain,
-              inspectBrk) {
-    super(url, importAttributes, phase, isMain, inspectBrk, true);
+              inspectBrk, requestType) {
+    super(loader, url, importAttributes, phase, isMain, inspectBrk);
 
-    this.#loader = loader;
     this.module = moduleWrap;
 
     assert(this.module instanceof ModuleWrap);
     this.linked = undefined;
     this.type = importAttributes.type;
     if (phase === kEvaluationPhase) {
-      this.#link();
+      this.linked = this.link(requestType);
     }
   }
 
   /**
-   * Ensure that this ModuleJob is at the required phase
-   * @param {number} phase
+   * @param {ModuleRequestType} requestType Type of the module request.
+   * @returns {ModuleJobBase[]}
    */
-  ensurePhase(phase) {
-    if (this.phase < phase) {
-      this.phase = phase;
-      this.#link();
-    }
-  }
-
-  #link() {
-    // Store itself into the cache first before linking in case there are circular
-    // references in the linking.
-    this.#loader.loadCache.set(this.url, this.type, this);
-    try {
-      const moduleRequests = this.module.getModuleRequests();
-      // Modules should be aligned with the moduleRequests array in order.
-      const modules = Array(moduleRequests.length);
-      const evaluationDepJobs = Array(moduleRequests.length);
-      let j = 0;
-      for (let i = 0; i < moduleRequests.length; ++i) {
-        const { specifier, attributes, phase } = moduleRequests[i];
-        const job = this.#loader.getModuleJobForRequire(specifier, this.url, attributes, phase);
-        modules[i] = job.module;
-        if (phase === kEvaluationPhase) {
-          evaluationDepJobs[j++] = job;
-        }
-      }
-      evaluationDepJobs.length = j;
-      this.module.link(modules);
-      this.linked = evaluationDepJobs;
-    } finally {
-      // Restore it - if it succeeds, we'll reset in the caller; Otherwise it's
-      // not cached and if the error is caught, subsequent attempt would still fail.
-      this.#loader.loadCache.delete(this.url, this.type);
-    }
+  link(requestType) {
+    // Synchronous linking is always used for ModuleJobSync.
+    return this.syncLink(requestType);
   }
 
   get modulePromise() {
@@ -472,7 +498,7 @@ class ModuleJobSync extends ModuleJobBase {
       }
       return { __proto__: null, module: this.module };
     } else if (status === kInstantiated) {
-      // The evaluation may have been canceled because instantiateSync() detected TLA first.
+      // The evaluation may have been canceled because instantiate() detected TLA first.
       // But when it is imported again, it's fine to re-evaluate it asynchronously.
       const timeout = -1;
       const breakOnSigint = false;
@@ -490,7 +516,7 @@ class ModuleJobSync extends ModuleJobBase {
     debug('ModuleJobSync.runSync()', this.module);
     assert(this.phase === kEvaluationPhase);
     // TODO(joyeecheung): add the error decoration logic from the async instantiate.
-    this.module.hasAsyncGraph = this.module.instantiateSync();
+    this.module.instantiate();
     // If --experimental-print-required-tla is true, proceeds to evaluation even
     // if it's async because we want to search for the TLA and help users locate
     // them.
