@@ -22,6 +22,7 @@ const {
   StringPrototypeStartsWith,
   encodeURIComponent,
 } = primordials;
+const assert = require('internal/assert');
 const internalFS = require('internal/fs/utils');
 const { BuiltinModule } = require('internal/bootstrap/realm');
 const { realpathSync } = require('fs');
@@ -41,6 +42,7 @@ const {
   ERR_INVALID_MODULE_SPECIFIER,
   ERR_INVALID_PACKAGE_CONFIG,
   ERR_INVALID_PACKAGE_TARGET,
+  ERR_INVALID_URL,
   ERR_MODULE_NOT_FOUND,
   ERR_PACKAGE_IMPORT_NOT_DEFINED,
   ERR_PACKAGE_PATH_NOT_EXPORTED,
@@ -117,18 +119,17 @@ function emitInvalidSegmentDeprecation(target, request, match, pjsonUrl, interna
  * Emits a deprecation warning if the given URL is a module and
  * the package.json file does not define a "main" or "exports" field.
  * @param {URL} url - The URL of the module being resolved.
- * @param {URL} packageJSONUrl - The URL of the package.json file for the module.
+ * @param {string} path - The path of the module being resolved.
+ * @param {string} pkgPath - The path of the parent dir of the package.json file for the module.
  * @param {string | URL} [base] - The base URL for the module being resolved.
  * @param {string} [main] - The "main" field from the package.json file.
  */
-function emitLegacyIndexDeprecation(url, packageJSONUrl, base, main) {
+function emitLegacyIndexDeprecation(url, path, pkgPath, base, main) {
   if (process.noDeprecation) {
     return;
   }
   const format = defaultGetFormatWithoutErrors(url);
   if (format !== 'module') { return; }
-  const path = fileURLToPath(url);
-  const pkgPath = fileURLToPath(new URL('.', packageJSONUrl));
   const basePath = fileURLToPath(base);
   if (!main) {
     process.emitWarning(
@@ -196,20 +197,19 @@ const legacyMainResolveExtensionsIndexes = {
  * @returns {URL}
  */
 function legacyMainResolve(packageJSONUrl, packageConfig, base) {
-  const packageJsonUrlString = packageJSONUrl.href;
-
-  if (typeof packageJsonUrlString !== 'string') {
-    throw new ERR_INVALID_ARG_TYPE('packageJSONUrl', ['URL'], packageJSONUrl);
-  }
+  assert(isURL(packageJSONUrl));
+  const pkgPath = fileURLToPath(new URL('.', packageJSONUrl));
 
   const baseStringified = isURL(base) ? base.href : base;
 
-  const resolvedOption = FSLegacyMainResolve(packageJsonUrlString, packageConfig.main, baseStringified);
+  const resolvedOption = FSLegacyMainResolve(pkgPath, packageConfig.main, baseStringified);
 
-  const baseUrl = resolvedOption <= legacyMainResolveExtensionsIndexes.kResolvedByMainIndexNode ? `./${packageConfig.main}` : '';
-  const resolvedUrl = new URL(baseUrl + legacyMainResolveExtensions[resolvedOption], packageJSONUrl);
+  const maybeMain = resolvedOption <= legacyMainResolveExtensionsIndexes.kResolvedByMainIndexNode ?
+    packageConfig.main || './' : '';
+  const resolvedPath = resolve(pkgPath, maybeMain + legacyMainResolveExtensions[resolvedOption]);
+  const resolvedUrl = pathToFileURL(resolvedPath);
 
-  emitLegacyIndexDeprecation(resolvedUrl, packageJSONUrl, base, packageConfig.main);
+  emitLegacyIndexDeprecation(resolvedUrl, resolvedPath, pkgPath, base, packageConfig.main);
 
   return resolvedUrl;
 }
@@ -228,34 +228,52 @@ const encodedSepRegEx = /%2F|%5C/i;
  */
 function finalizeResolution(resolved, base, preserveSymlinks) {
   if (RegExpPrototypeExec(encodedSepRegEx, resolved.pathname) !== null) {
+    let basePath;
+    try {
+      basePath = fileURLToPath(base);
+    } catch {
+      basePath = base;
+    }
     throw new ERR_INVALID_MODULE_SPECIFIER(
       resolved.pathname, 'must not include encoded "/" or "\\" characters',
-      fileURLToPath(base));
+      basePath);
   }
 
   let path;
   try {
     path = fileURLToPath(resolved);
   } catch (err) {
-    const { setOwnProperty } = require('internal/util');
     setOwnProperty(err, 'input', `${resolved}`);
     setOwnProperty(err, 'module', `${base}`);
     throw err;
   }
 
-  const stats = internalFsBinding.internalModuleStat(StringPrototypeEndsWith(path, '/') ?
-    StringPrototypeSlice(path, -1) : path);
+  const stats = internalFsBinding.internalModuleStat(
+    StringPrototypeEndsWith(internalFsBinding, path, '/') ? StringPrototypeSlice(path, -1) : path,
+  );
 
   // Check for stats.isDirectory()
   if (stats === 1) {
-    throw new ERR_UNSUPPORTED_DIR_IMPORT(path, fileURLToPath(base), String(resolved));
+    let basePath;
+    try {
+      basePath = fileURLToPath(base);
+    } catch {
+      basePath = base;
+    }
+    throw new ERR_UNSUPPORTED_DIR_IMPORT(path, basePath, String(resolved));
   } else if (stats !== 0) {
     // Check for !stats.isFile()
     if (process.env.WATCH_REPORT_DEPENDENCIES && process.send) {
       process.send({ 'watch:require': [path || resolved.pathname] });
     }
+    let basePath;
+    try {
+      basePath = fileURLToPath(base);
+    } catch {
+      basePath = base;
+    }
     throw new ERR_MODULE_NOT_FOUND(
-      path || resolved.pathname, base && fileURLToPath(base), resolved);
+      path || resolved.pathname, basePath, resolved);
   }
 
   if (!preserveSymlinks) {
@@ -338,7 +356,6 @@ function invalidPackageTarget(
 
 const invalidSegmentRegEx = /(^|\\|\/)((\.|%2e)(\.|%2e)?|(n|%6e|%4e)(o|%6f|%4f)(d|%64|%44)(e|%65|%45)(_|%5f)(m|%6d|%4d)(o|%6f|%4f)(d|%64|%44)(u|%75|%55)(l|%6c|%4c)(e|%65|%45)(s|%73|%53))?(\\|\/|$)/i;
 const deprecatedInvalidSegmentRegEx = /(^|\\|\/)((\.|%2e)(\.|%2e)?|(n|%6e|%4e)(o|%6f|%4f)(d|%64|%44)(e|%65|%45)(_|%5f)(m|%6d|%4d)(o|%6f|%4f)(d|%64|%44)(u|%75|%55)(l|%6c|%4c)(e|%65|%45)(s|%73|%53))(\\|\/|$)/i;
-const invalidPackageNameRegEx = /^\.|%|\\/;
 const patternRegEx = /\*/g;
 
 /**
@@ -373,8 +390,9 @@ function resolvePackageTargetString(
   }
 
   if (!StringPrototypeStartsWith(target, './')) {
-    if (internal && !StringPrototypeStartsWith(target, '../') &&
-        !StringPrototypeStartsWith(target, '/')) {
+    if (internal &&
+        target[0] !== '/' &&
+        !StringPrototypeStartsWith(target, '../')) {
       // No need to convert target to string, since it's already presumed to be
       if (!URLCanParse(target)) {
         const exportTarget = pattern ?
@@ -533,6 +551,7 @@ function resolvePackageTarget(packageJSONUrl, target, subpath, packageSubpath,
  * @param {import('internal/modules/esm/package_config.js').PackageConfig['exports']} exports
  * @param {URL} packageJSONUrl The URL of the package.json file.
  * @param {string | URL | undefined} base The base URL.
+ * @returns {boolean}
  */
 function isConditionalExportsMainSugar(exports, packageJSONUrl, base) {
   if (typeof exports === 'string' || ArrayIsArray(exports)) { return true; }
@@ -648,7 +667,7 @@ function packageExportsResolve(
  * @param {string} a - The first string to compare.
  * @param {string} b - The second string to compare.
  * @returns {number} - A negative number if `a` should come before `b`, a positive number if `a` should come after `b`,
- * or 0 if they are equal.
+ *   or 0 if they are equal.
  */
 function patternKeyCompare(a, b) {
   const aPatternIndex = StringPrototypeIndexOf(a, '*');
@@ -733,44 +752,6 @@ function packageImportsResolve(name, base, conditions) {
   throw importNotDefined(name, packageJSONUrl, base);
 }
 
-/**
- * Parse a package name from a specifier.
- * @param {string} specifier - The import specifier.
- * @param {string | URL | undefined} base - The parent URL.
- */
-function parsePackageName(specifier, base) {
-  let separatorIndex = StringPrototypeIndexOf(specifier, '/');
-  let validPackageName = true;
-  let isScoped = false;
-  if (specifier[0] === '@') {
-    isScoped = true;
-    if (separatorIndex === -1 || specifier.length === 0) {
-      validPackageName = false;
-    } else {
-      separatorIndex = StringPrototypeIndexOf(
-        specifier, '/', separatorIndex + 1);
-    }
-  }
-
-  const packageName = separatorIndex === -1 ?
-    specifier : StringPrototypeSlice(specifier, 0, separatorIndex);
-
-  // Package name cannot have leading . and cannot have percent-encoding or
-  // \\ separators.
-  if (RegExpPrototypeExec(invalidPackageNameRegEx, packageName) !== null) {
-    validPackageName = false;
-  }
-
-  if (!validPackageName) {
-    throw new ERR_INVALID_MODULE_SPECIFIER(
-      specifier, 'is not a valid package name', fileURLToPath(base));
-  }
-
-  const packageSubpath = '.' + (separatorIndex === -1 ? '' :
-    StringPrototypeSlice(specifier, separatorIndex));
-
-  return { packageName, packageSubpath, isScoped };
-}
 
 /**
  * Resolves a package specifier to a URL.
@@ -785,63 +766,30 @@ function packageResolve(specifier, base, conditions) {
     return new URL('node:' + specifier);
   }
 
-  const { packageName, packageSubpath, isScoped } =
-    parsePackageName(specifier, base);
+  const { packageJSONUrl, packageJSONPath, packageSubpath } = packageJsonReader.getPackageJSONURL(specifier, base);
 
-  // ResolveSelf
-  const packageConfig = packageJsonReader.getPackageScopeConfig(base);
-  if (packageConfig.exists) {
-    const packageJSONUrl = pathToFileURL(packageConfig.pjsonPath);
-    if (packageConfig.exports != null && packageConfig.name === packageName) {
-      return packageExportsResolve(
-        packageJSONUrl, packageSubpath, packageConfig, base, conditions);
-    }
+  const packageConfig = packageJsonReader.read(packageJSONPath, { __proto__: null, specifier, base, isESM: true });
+
+  // Package match.
+  if (packageConfig.exports != null) {
+    return packageExportsResolve(
+      packageJSONUrl, packageSubpath, packageConfig, base, conditions);
+  }
+  if (packageSubpath === '.') {
+    return legacyMainResolve(
+      packageJSONUrl,
+      packageConfig,
+      base,
+    );
   }
 
-  let packageJSONUrl =
-    new URL('./node_modules/' + packageName + '/package.json', base);
-  let packageJSONPath = fileURLToPath(packageJSONUrl);
-  let lastPath;
-  do {
-    const stat = internalFsBinding.internalModuleStat(
-      StringPrototypeSlice(packageJSONPath, 0, packageJSONPath.length - 13),
-    );
-    // Check for !stat.isDirectory()
-    if (stat !== 1) {
-      lastPath = packageJSONPath;
-      packageJSONUrl = new URL((isScoped ?
-        '../../../../node_modules/' : '../../../node_modules/') +
-        packageName + '/package.json', packageJSONUrl);
-      packageJSONPath = fileURLToPath(packageJSONUrl);
-      continue;
-    }
-
-    // Package match.
-    const packageConfig = packageJsonReader.read(packageJSONPath, { __proto__: null, specifier, base, isESM: true });
-    if (packageConfig.exports != null) {
-      return packageExportsResolve(
-        packageJSONUrl, packageSubpath, packageConfig, base, conditions);
-    }
-    if (packageSubpath === '.') {
-      return legacyMainResolve(
-        packageJSONUrl,
-        packageConfig,
-        base,
-      );
-    }
-
-    return new URL(packageSubpath, packageJSONUrl);
-    // Cross-platform root check.
-  } while (packageJSONPath.length !== lastPath.length);
-
-  // eslint can't handle the above code.
-  // eslint-disable-next-line no-unreachable
-  throw new ERR_MODULE_NOT_FOUND(packageName, fileURLToPath(base), null);
+  return new URL(packageSubpath, packageJSONUrl);
 }
 
 /**
  * Checks if a specifier is a bare specifier.
  * @param {string} specifier - The specifier to check.
+ * @returns {boolean}
  */
 function isBareSpecifier(specifier) {
   return specifier[0] && specifier[0] !== '/' && specifier[0] !== '.';
@@ -850,6 +798,7 @@ function isBareSpecifier(specifier) {
 /**
  * Determines whether a specifier is a relative path.
  * @param {string} specifier - The specifier to check.
+ * @returns {boolean}
  */
 function isRelativeSpecifier(specifier) {
   if (specifier[0] === '.') {
@@ -864,6 +813,7 @@ function isRelativeSpecifier(specifier) {
 /**
  * Determines whether a specifier should be treated as a relative or absolute path.
  * @param {string} specifier - The specifier to check.
+ * @returns {boolean}
  */
 function shouldBeTreatedAsRelativeOrAbsolutePath(specifier) {
   if (specifier === '') { return false; }
@@ -877,6 +827,7 @@ function shouldBeTreatedAsRelativeOrAbsolutePath(specifier) {
  * @param {string | URL | undefined} base - The base URL to resolve against.
  * @param {Set<string>} conditions - An object containing environment conditions.
  * @param {boolean} preserveSymlinks - Whether to preserve symlinks in the resolved URL.
+ * @returns {URL}
  */
 function moduleResolve(specifier, base, conditions, preserveSymlinks) {
   const protocol = typeof base === 'string' ?
@@ -897,12 +848,13 @@ function moduleResolve(specifier, base, conditions, preserveSymlinks) {
   } else if (protocol === 'file:' && specifier[0] === '#') {
     resolved = packageImportsResolve(specifier, base, conditions);
   } else {
-    try {
-      resolved = new URL(specifier);
-    } catch (cause) {
+    const url = URLParse(specifier);
+    if (url !== null) {
+      resolved = url;
+    } else {
       if (isData && !BuiltinModule.canBeRequiredWithoutScheme(specifier)) {
         const error = new ERR_UNSUPPORTED_RESOLVE_REQUEST(specifier, base);
-        setOwnProperty(error, 'cause', cause);
+        setOwnProperty(error, 'cause', new ERR_INVALID_URL(specifier));
         throw error;
       }
       resolved = packageResolve(specifier, base, conditions);
@@ -972,6 +924,7 @@ function resolveAsCommonJS(specifier, parentURL) {
 /**
  * Validate user-input in `context` supplied by a custom loader.
  * @param {string | URL | undefined} parentURL - The parent URL.
+ * @returns {void}
  */
 function throwIfInvalidParentURL(parentURL) {
   if (parentURL === undefined) {
@@ -986,9 +939,10 @@ function throwIfInvalidParentURL(parentURL) {
  * Resolves the given specifier using the provided context, which includes the parent URL and conditions.
  * Attempts to resolve the specifier and returns the resulting URL and format.
  * @param {string} specifier - The specifier to resolve.
- * @param {object} [context={}] - The context object containing the parent URL and conditions.
+ * @param {object} [context] - The context object containing the parent URL and conditions.
  * @param {string} [context.parentURL] - The URL of the parent module.
  * @param {string[]} [context.conditions] - The conditions for resolving the specifier.
+ * @returns {{url: string, format?: string}}
  */
 function defaultResolve(specifier, context = {}) {
   let { parentURL, conditions } = context;
@@ -1087,10 +1041,11 @@ module.exports = {
   decorateErrorWithCommonJSHints,
   defaultResolve,
   encodedSepRegEx,
+  legacyMainResolve,
   packageExportsResolve,
   packageImportsResolve,
+  packageResolve,
   throwIfInvalidParentURL,
-  legacyMainResolve,
 };
 
 // cycle
