@@ -32,6 +32,7 @@ const {
   Promise,
   SafeSet,
   Symbol,
+  SymbolAsyncDispose,
   SymbolAsyncIterator,
   SymbolSpecies,
   TypedArrayPrototypeSet,
@@ -43,14 +44,11 @@ Readable.ReadableState = ReadableState;
 const EE = require('events');
 const { Stream, prependListener } = require('internal/streams/legacy');
 const { Buffer } = require('buffer');
-const {
-  SymbolAsyncDispose,
-} = require('internal/util');
 
 const {
   addAbortSignal,
 } = require('internal/streams/add-abort-signal');
-const eos = require('internal/streams/end-of-stream');
+const { eos } = require('internal/streams/end-of-stream');
 
 let debug = require('internal/util/debuglog').debuglog('stream', (fn) => {
   debug = fn;
@@ -266,10 +264,10 @@ function ReadableState(options, stream, isDuplex) {
 
   // Object stream flag. Used to make read(n) ignore n and to
   // make all the buffer merging and length checks go away.
-  if (options && options.objectMode)
+  if (options?.objectMode)
     this[kState] |= kObjectMode;
 
-  if (isDuplex && options && options.readableObjectMode)
+  if (isDuplex && options?.readableObjectMode)
     this[kState] |= kObjectMode;
 
   // The point at which it stops calling _read() to fill the buffer
@@ -305,7 +303,7 @@ function ReadableState(options, stream, isDuplex) {
   // type: null | Writable | Set<Writable>.
   this.awaitDrainWriters = null;
 
-  if (options && options.encoding) {
+  if (options?.encoding) {
     this.decoder = new StringDecoder(options.encoding);
     this.encoding = options.encoding;
   }
@@ -371,13 +369,13 @@ Readable.prototype[EE.captureRejectionSymbol] = function(err) {
   this.destroy(err);
 };
 
-Readable.prototype[SymbolAsyncDispose] = function() {
+Readable.prototype[SymbolAsyncDispose] = async function() {
   let error;
   if (!this.destroyed) {
     error = this.readableEnded ? null : new AbortError();
     this.destroy(error);
   }
-  return new Promise((resolve, reject) => eos(this, (err) => (err && err !== error ? reject(err) : resolve(null))));
+  await new Promise((resolve, reject) => eos(this, (err) => (err && err !== error ? reject(err) : resolve(null))));
 };
 
 // Manually shove something into the read() buffer.
@@ -412,7 +410,7 @@ function readableAddChunkUnshiftByteMode(stream, state, chunk, encoding) {
   }
 
   if (typeof chunk === 'string') {
-    encoding = encoding || state.defaultEncoding;
+    encoding ||= state.defaultEncoding;
     if (state.encoding !== encoding) {
       if (state.encoding) {
         // When unshifting, if state.encoding is set, we have to save
@@ -468,7 +466,7 @@ function readableAddChunkPushByteMode(stream, state, chunk, encoding) {
   }
 
   if (typeof chunk === 'string') {
-    encoding = encoding || state.defaultEncoding;
+    encoding ||= state.defaultEncoding;
     if (state.encoding !== encoding) {
       chunk = Buffer.from(chunk, encoding);
       encoding = '';
@@ -791,7 +789,7 @@ function onEofChunk(stream, state) {
   const decoder = (state[kState] & kDecoder) !== 0 ? state[kDecoderValue] : null;
   if (decoder) {
     const chunk = decoder.end();
-    if (chunk && chunk.length) {
+    if (chunk?.length) {
       state.buffer.push(chunk);
       state.length += (state[kState] & kObjectMode) !== 0 ? 1 : chunk.length;
     }
@@ -855,7 +853,7 @@ function emitReadable_(stream) {
 // However, if we're not ended, or reading, and the length < hwm,
 // then go ahead and try to read some more preemptively.
 function maybeReadMore(stream, state) {
-  if ((state[kState] & (kReadingMore | kConstructed)) === kConstructed) {
+  if ((state[kState] & (kReadingMore | kReading | kConstructed)) === kConstructed) {
     state[kState] |= kReadingMore;
     process.nextTick(maybeReadMore_, stream, state);
   }
@@ -1006,10 +1004,15 @@ Readable.prototype.pipe = function(dest, pipeOpts) {
   src.on('data', ondata);
   function ondata(chunk) {
     debug('ondata');
-    const ret = dest.write(chunk);
-    debug('dest.write', ret);
-    if (ret === false) {
-      pause();
+    try {
+      const ret = dest.write(chunk);
+      debug('dest.write', ret);
+
+      if (ret === false) {
+        pause();
+      }
+    } catch (error) {
+      dest.destroy(error);
     }
   }
 
@@ -1461,7 +1464,7 @@ ObjectDefineProperties(Readable.prototype, {
     __proto__: null,
     enumerable: false,
     get: function() {
-      return this._readableState && this._readableState.buffer;
+      return this._readableState?.buffer;
     },
   },
 
@@ -1602,7 +1605,7 @@ function fromList(n, state) {
         buf[idx++] = null;
       }
     } else if (len - idx === 0) {
-      ret = Buffer.alloc(0);
+      ret = new FastBuffer();
     } else if (len - idx === 1) {
       ret = buf[idx];
       buf[idx++] = null;
