@@ -46,7 +46,7 @@ const EE = require('events');
 const Stream = require('internal/streams/legacy').Stream;
 const { Buffer } = require('buffer');
 const destroyImpl = require('internal/streams/destroy');
-const eos = require('internal/streams/end-of-stream');
+const { eos } = require('internal/streams/end-of-stream');
 
 const {
   addAbortSignal,
@@ -1149,13 +1149,13 @@ Writable.toWeb = function(streamWritable) {
   return lazyWebStreams().newWritableStreamFromStreamWritable(streamWritable);
 };
 
-Writable.prototype[SymbolAsyncDispose] = function() {
+Writable.prototype[SymbolAsyncDispose] = async function() {
   let error;
   if (!this.destroyed) {
     error = this.writableFinished ? null : new AbortError();
     this.destroy(error);
   }
-  return new Promise((resolve, reject) =>
+  await new Promise((resolve, reject) =>
     eos(this, (err) => (err && err.name !== 'AbortError' ? reject(err) : resolve(null))),
   );
 };
