@@ -19,11 +19,11 @@ const inspectOptions = { __proto__: null, colors: false, breakLength: Infinity }
 const HOSTNAME = hostname();
 
 function escapeAttribute(s = '') {
-  return escapeContent(RegExpPrototypeSymbolReplace(/"/g, RegExpPrototypeSymbolReplace(/\n/g, s, ''), '&quot;'));
+  return escapeContent(RegExpPrototypeSymbolReplace(/"/g, RegExpPrototypeSymbolReplace(/\n/g, s, '&#10;'), '&quot;'));
 }
 
 function escapeContent(s = '') {
-  return RegExpPrototypeSymbolReplace(/</g, RegExpPrototypeSymbolReplace(/&/g, s, '&amp;'), '&lt;');
+  return RegExpPrototypeSymbolReplace(/</g, RegExpPrototypeSymbolReplace(/(&)(?!#\d{1,7};)/g, s, '&amp;'), '&lt;');
 }
 
 function escapeComment(s = '') {
@@ -116,6 +116,9 @@ module.exports = async function* junitReporter(source) {
         } else {
           currentTest.tag = 'testcase';
           currentTest.attrs.classname = event.data.classname ?? 'test';
+          if (event.data.file) {
+            currentTest.attrs.file = event.data.file;
+          }
           if (event.data.skip) {
             ArrayPrototypePush(currentTest.children, {
               __proto__: null, nesting: event.data.nesting + 1, tag: 'skipped',
@@ -134,7 +137,7 @@ module.exports = async function* junitReporter(source) {
               __proto__: null,
               nesting: event.data.nesting + 1,
               tag: 'failure',
-              attrs: { __proto__: null, type: error?.failureType || error?.code, message: error?.message ?? '' },
+              attrs: { __proto__: null, type: error?.failureType || error?.code, message: error?.message.trim() ?? '' },
               children: [inspectWithNoCustomRetry(error, inspectOptions)],
             });
             currentTest.failures = 1;
