@@ -13,7 +13,6 @@ const {
   MathMax,
   Number,
   NumberPrototypeToFixed,
-  ObjectDefineProperty,
   ObjectSeal,
   Promise,
   PromisePrototypeThen,
@@ -55,6 +54,7 @@ const {
 const {
   kEmptyObject,
   once: runOnce,
+  setOwnProperty,
 } = require('internal/util');
 const { isPromise } = require('internal/util/types');
 const {
@@ -71,6 +71,7 @@ const {
 } = require('timers');
 const { TIMEOUT_MAX } = require('internal/timers');
 const { fileURLToPath } = require('internal/url');
+const { relative } = require('path');
 const { availableParallelism } = require('os');
 const { innerOk } = require('internal/assert/utils');
 const { bigint: hrtime } = process.hrtime;
@@ -132,20 +133,14 @@ function stopTest(timeout, signal) {
   if (timeout === kDefaultTimeout) {
     disposeFunction = abortListener[SymbolDispose];
   } else {
-    timer = setTimeout(() => deferred.resolve(), timeout);
+    timer = setTimeout(deferred.resolve, timeout);
     timer.unref();
-
-    ObjectDefineProperty(deferred, 'promise', {
-      __proto__: null,
-      configurable: true,
-      writable: true,
-      value: PromisePrototypeThen(deferred.promise, () => {
-        throw new ERR_TEST_FAILURE(
-          `test timed out after ${timeout}ms`,
-          kTestTimeoutFailure,
-        );
-      }),
-    });
+    setOwnProperty(deferred, 'promise', PromisePrototypeThen(deferred.promise, () => {
+      throw new ERR_TEST_FAILURE(
+        `test timed out after ${timeout}ms`,
+        kTestTimeoutFailure,
+      );
+    }));
 
     disposeFunction = () => {
       abortListener[SymbolDispose]();
@@ -153,12 +148,7 @@ function stopTest(timeout, signal) {
     };
   }
 
-  ObjectDefineProperty(deferred.promise, SymbolDispose, {
-    __proto__: null,
-    configurable: true,
-    writable: true,
-    value: disposeFunction,
-  });
+  setOwnProperty(deferred.promise, SymbolDispose, disposeFunction);
   return deferred.promise;
 }
 
@@ -290,6 +280,10 @@ class TestContext {
     return this.#test.passed;
   }
 
+  get attempt() {
+    return this.#test.attempt ?? 0;
+  }
+
   diagnostic(message) {
     this.#test.diagnostic(message);
   }
@@ -537,6 +531,7 @@ class Test extends AsyncResource {
       this.childNumber = 0;
       this.timeout = kDefaultTimeout;
       this.entryFile = entryFile;
+      this.testDisambiguator = new SafeMap();
     } else {
       const nesting = parent.parent === null ? parent.nesting :
         parent.nesting + 1;
@@ -641,11 +636,13 @@ class Test extends AsyncResource {
     this.expectedAssertions = plan;
     this.cancelled = false;
     this.skipped = skip !== undefined && skip !== false;
-    this.isTodo = todo !== undefined && todo !== false;
+    this.isTodo = (todo !== undefined && todo !== false) || this.parent?.isTodo;
     this.startTime = null;
     this.endTime = null;
     this.passed = false;
     this.error = null;
+    this.attempt = undefined;
+    this.passedAttempt = undefined;
     this.message = typeof skip === 'string' ? skip :
       typeof todo === 'string' ? todo : null;
     this.activeSubtests = 0;
@@ -690,6 +687,31 @@ class Test extends AsyncResource {
         this.loc.file = fileURLToPath(this.loc.file);
       }
     }
+
+    if (this.loc != null && this.root.harness.previousRuns != null) {
+      let testIdentifier = `${relative(this.config.cwd, this.loc.file)}:${this.loc.line}:${this.loc.column}`;
+      const disambiguator = this.root.testDisambiguator.get(testIdentifier);
+      if (disambiguator !== undefined) {
+        testIdentifier += `:(${disambiguator})`;
+        this.root.testDisambiguator.set(testIdentifier, disambiguator + 1);
+      } else {
+        this.root.testDisambiguator.set(testIdentifier, 1);
+      }
+      this.attempt = this.root.harness.previousRuns.length;
+      const previousAttempt = this.root.harness.previousRuns[this.attempt - 1]?.[testIdentifier];
+      if (previousAttempt != null) {
+        this.passedAttempt = previousAttempt.passed_on_attempt;
+        this.fn = () => {
+          for (let i = 0; i < (previousAttempt.children?.length ?? 0); i++) {
+            const child = previousAttempt.children[i];
+            this.createSubtest(Test, child.name, { __proto__: null }, noop, {
+              __proto__: null,
+              loc: [child.line, child.column, child.file],
+            }, noop).start();
+          }
+        };
+      }
+    }
   }
 
   applyFilters() {
@@ -969,7 +991,7 @@ class Test extends AsyncResource {
     if (this.root.harness.buildPromise || !this.parent.hasConcurrency()) {
       const deferred = PromiseWithResolvers();
 
-      deferred.test = this;
+      setOwnProperty(deferred, 'test', this);
       this.parent.addPendingSubtest(deferred);
       return deferred.promise;
     }
@@ -1329,6 +1351,12 @@ class Test extends AsyncResource {
     if (!this.passed) {
       details.error = this.error;
     }
+    if (this.attempt !== undefined) {
+      details.attempt = this.attempt;
+    }
+    if (this.passedAttempt !== undefined) {
+      details.passed_on_attempt = this.passedAttempt;
+    }
     return { __proto__: null, details, directive };
   }
 
@@ -1424,7 +1452,9 @@ class Suite extends Test {
   reportedType = 'suite';
   constructor(options) {
     super(options);
-    this.timeout = null;
+    if (options.timeout == null) {
+      this.timeout = null;
+    }
 
     if (this.config.testNamePatterns !== null &&
         this.config.testSkipPatterns !== null &&
@@ -1502,6 +1532,8 @@ class Suite extends Test {
 }
 
 function getFullName(test) {
+  if (test === test.root) return test.name;
+
   let fullName = test.name;
 
   for (let t = test.parent; t !== t.root; t = t.parent) {
