@@ -8,6 +8,8 @@ const {
   Error,
   ErrorCaptureStackTrace,
   FunctionPrototypeCall,
+  FunctionPrototypeSymbolHasInstance,
+  NumberParseInt,
   ObjectDefineProperties,
   ObjectDefineProperty,
   ObjectFreeze,
@@ -33,13 +35,17 @@ const {
   SafeSet,
   SafeWeakMap,
   SafeWeakRef,
+  StringPrototypeIncludes,
   StringPrototypeReplace,
+  StringPrototypeSlice,
   StringPrototypeToLowerCase,
   StringPrototypeToUpperCase,
   Symbol,
   SymbolFor,
+  SymbolPrototypeGetDescription,
   SymbolReplace,
   SymbolSplit,
+  globalThis,
 } = primordials;
 
 const {
@@ -47,6 +53,7 @@ const {
     ERR_NO_CRYPTO,
     ERR_NO_TYPESCRIPT,
     ERR_UNKNOWN_SIGNAL,
+    ERR_WEBASSEMBLY_NOT_SUPPORTED,
   },
   isErrorStackTraceLimitWritable,
   overrideStackTrace,
@@ -54,7 +61,9 @@ const {
 } = require('internal/errors');
 const { signals } = internalBinding('constants').os;
 const {
+  constructSharedArrayBuffer,
   guessHandleType: _guessHandleType,
+  defineLazyProperties,
   privateSymbols: {
     arrow_message_private_symbol,
     decorated_private_symbol,
@@ -63,6 +72,7 @@ const {
 } = internalBinding('util');
 const { isNativeError, isPromise } = internalBinding('types');
 const { getOptionValue } = require('internal/options');
+const assert = require('internal/assert');
 const { encodings } = internalBinding('string_decoder');
 
 const noCrypto = !process.versions.openssl;
@@ -90,7 +100,7 @@ function isError(e) {
   // An error could be an instance of Error while not being a native error
   // or could be from a different realm and not be instance of Error but still
   // be a native error.
-  return isNativeError(e) || e instanceof Error;
+  return isNativeError(e) || FunctionPrototypeSymbolHasInstance(Error, e);
 }
 
 // Keep a list of deprecation codes that have been warned on so we only warn on
@@ -104,8 +114,8 @@ function getDeprecationWarningEmitter(
   shouldEmitWarning = () => true,
 ) {
   let warned = false;
-  return function() {
-    if (!warned && shouldEmitWarning()) {
+  return function(arg) {
+    if (!warned && shouldEmitWarning(arg)) {
       warned = true;
       if (code === 'ExperimentalWarning') {
         process.emitWarning(msg, code, deprecated);
@@ -129,6 +139,17 @@ function isPendingDeprecation() {
     !getOptionValue('--no-deprecation');
 }
 
+function deprecateProperty(key, msg, code, isPendingDeprecation) {
+  const emitDeprecationWarning = getDeprecationWarningEmitter(
+    code, msg, undefined, false, isPendingDeprecation,
+  );
+  return (options) => {
+    if (key in options) {
+      emitDeprecationWarning();
+    }
+  };
+}
+
 // Internal deprecator for pending --pending-deprecation. This can be invoked
 // at snapshot building time as the warning permission is only queried at
 // run time.
@@ -140,13 +161,19 @@ function pendingDeprecate(fn, msg, code) {
     emitDeprecationWarning();
     return ReflectApply(fn, this, args);
   }
+
+  ObjectDefineProperty(deprecated, 'length', {
+    __proto__: null,
+    ...ObjectGetOwnPropertyDescriptor(fn, 'length'),
+  });
+
   return deprecated;
 }
 
 // Mark that a method should not be used.
 // Returns a modified function which warns once by default.
 // If --no-deprecation is set, then it is a no-op.
-function deprecate(fn, msg, code, useEmitSync) {
+function deprecate(fn, msg, code, useEmitSync, modifyPrototype = true) {
   // Lazy-load to avoid a circular dependency.
   if (validateString === undefined)
     ({ validateString } = require('internal/validators'));
@@ -159,7 +186,6 @@ function deprecate(fn, msg, code, useEmitSync) {
   );
 
   function deprecated(...args) {
-    // TODO(joyeecheung): use getOptionValue('--no-deprecation') instead.
     if (!process.noDeprecation) {
       emitDeprecationWarning();
     }
@@ -169,18 +195,35 @@ function deprecate(fn, msg, code, useEmitSync) {
     return ReflectApply(fn, this, args);
   }
 
-  // The wrapper will keep the same prototype as fn to maintain prototype chain
-  ObjectSetPrototypeOf(deprecated, fn);
-  if (fn.prototype) {
-    // Setting this (rather than using Object.setPrototype, as above) ensures
-    // that calling the unwrapped constructor gives an instanceof the wrapped
-    // constructor.
-    deprecated.prototype = fn.prototype;
+  if (modifyPrototype) {
+    // The wrapper will keep the same prototype as fn to maintain prototype chain
+    // Modifying the prototype does alter the object chains, and as observed in
+    // most cases, it slows the code.
+    ObjectSetPrototypeOf(deprecated, fn);
+    if (fn.prototype) {
+      // Setting this (rather than using Object.setPrototype, as above) ensures
+      // that calling the unwrapped constructor gives an instanceof the wrapped
+      // constructor.
+      deprecated.prototype = fn.prototype;
+    }
+
+    ObjectDefineProperty(deprecated, 'length', {
+      __proto__: null,
+      ...ObjectGetOwnPropertyDescriptor(fn, 'length'),
+    });
   }
 
   return deprecated;
 }
 
+function deprecateInstantiation(target, code, ...args) {
+  assert(typeof code === 'string');
+
+  getDeprecationWarningEmitter(code, `Instantiating ${target.name} without the 'new' keyword has been deprecated.`, target)();
+
+  return ReflectConstruct(target, args);
+}
+
 function decorateErrorStack(err) {
   if (!(isError(err) && err.stack) || err[decorated_private_symbol])
     return;
@@ -201,6 +244,8 @@ function assertCrypto() {
 function assertTypeScript() {
   if (noTypeScript)
     throw new ERR_NO_TYPESCRIPT();
+  if (globalThis.WebAssembly === undefined)
+    throw new ERR_WEBASSEMBLY_NOT_SUPPORTED('TypeScript');
 }
 
 /**
@@ -601,46 +646,6 @@ function exposeGetterAndSetter(target, name, getter, setter = undefined) {
   });
 }
 
-function defineLazyProperties(target, id, keys, enumerable = true) {
-  const descriptors = { __proto__: null };
-  let mod;
-  for (let i = 0; i < keys.length; i++) {
-    const key = keys[i];
-    let lazyLoadedValue;
-    function set(value) {
-      ObjectDefineProperty(target, key, {
-        __proto__: null,
-        writable: true,
-        value,
-      });
-    }
-    ObjectDefineProperty(set, 'name', {
-      __proto__: null,
-      value: `set ${key}`,
-    });
-    function get() {
-      mod ??= require(id);
-      if (lazyLoadedValue === undefined) {
-        lazyLoadedValue = mod[key];
-        set(lazyLoadedValue);
-      }
-      return lazyLoadedValue;
-    }
-    ObjectDefineProperty(get, 'name', {
-      __proto__: null,
-      value: `get ${key}`,
-    });
-    descriptors[key] = {
-      __proto__: null,
-      configurable: true,
-      enumerable,
-      get,
-      set,
-    };
-  }
-  ObjectDefineProperties(target, descriptors);
-}
-
 function defineReplaceableLazyAttribute(target, id, keys, writable = true, check) {
   let mod;
   for (let i = 0; i < keys.length; i++) {
@@ -825,6 +830,59 @@ function setupCoverageHooks(dir) {
   return coverageDirectory;
 }
 
+// Returns the number of ones in the binary representation of the decimal
+// number.
+function countBinaryOnes(n) {
+  // Count the number of bits set in parallel, which is faster than looping
+  n = n - ((n >>> 1) & 0x55555555);
+  n = (n & 0x33333333) + ((n >>> 2) & 0x33333333);
+  return ((n + (n >>> 4) & 0xF0F0F0F) * 0x1010101) >>> 24;
+}
+
+function getCIDR(address, netmask, family) {
+  let ones = 0;
+  let split = '.';
+  let range = 10;
+  let groupLength = 8;
+  let hasZeros = false;
+  let lastPos = 0;
+
+  if (family === 'IPv6') {
+    split = ':';
+    range = 16;
+    groupLength = 16;
+  }
+
+  for (let i = 0; i < netmask.length; i++) {
+    if (netmask[i] !== split) {
+      if (i + 1 < netmask.length) {
+        continue;
+      }
+      i++;
+    }
+    const part = StringPrototypeSlice(netmask, lastPos, i);
+    lastPos = i + 1;
+    if (part !== '') {
+      if (hasZeros) {
+        if (part !== '0') {
+          return null;
+        }
+      } else {
+        const binary = NumberParseInt(part, range);
+        const binaryOnes = countBinaryOnes(binary);
+        ones += binaryOnes;
+        if (binaryOnes !== groupLength) {
+          if (StringPrototypeIncludes(binary.toString(2), '01')) {
+            return null;
+          }
+          hasZeros = true;
+        }
+      }
+    }
+  }
+
+  return `${address}/${ones}`;
+}
 
 const handleTypes = ['TCP', 'TTY', 'UDP', 'FILE', 'PIPE', 'UNKNOWN'];
 function guessHandleType(fd) {
@@ -869,11 +927,39 @@ const encodingsMap = { __proto__: null };
 for (let i = 0; i < encodings.length; ++i)
   encodingsMap[encodings[i]] = i;
 
+/**
+ * Reassigns the .name property of a function.
+ * Should be used when function can't be initially defined with desired name
+ * or when desired name should include `#`, `[`, `]`, etc.
+ * @param {string | symbol} name
+ * @param {Function} fn
+ * @param {object} [descriptor]
+ * @returns {Function} the same function, renamed
+ */
+function assignFunctionName(name, fn, descriptor = kEmptyObject) {
+  if (typeof name !== 'string') {
+    const symbolDescription = SymbolPrototypeGetDescription(name);
+    assert(symbolDescription !== undefined, 'Attempted to name function after descriptionless Symbol');
+    name = `[${symbolDescription}]`;
+  }
+  return ObjectDefineProperty(fn, 'name', {
+    __proto__: null,
+    writable: false,
+    enumerable: false,
+    configurable: true,
+    ...ObjectGetOwnPropertyDescriptor(fn, 'name'),
+    ...descriptor,
+    value: name,
+  });
+}
+
 module.exports = {
   getLazy,
   assertCrypto,
   assertTypeScript,
+  assignFunctionName,
   cachedResult,
+  constructSharedArrayBuffer,
   convertToValidSignal,
   createClassWrapper,
   decorateErrorStack,
@@ -881,6 +967,8 @@ module.exports = {
   defineLazyProperties,
   defineReplaceableLazyAttribute,
   deprecate,
+  deprecateInstantiation,
+  deprecateProperty,
   emitExperimentalWarning,
   encodingsMap,
   exposeInterface,
@@ -890,6 +978,7 @@ module.exports = {
   filterDuplicateStrings,
   filterOwnProperties,
   getConstructorOf,
+  getCIDR,
   getCWDURL,
   getInternalGlobal,
   getStructuredStack,
@@ -932,4 +1021,6 @@ module.exports = {
   setOwnProperty,
   pendingDeprecate,
   WeakReference,
+  isPendingDeprecation,
+  getDeprecationWarningEmitter,
 };
