@@ -1703,7 +1703,7 @@ function formatError(err, constructor, tag, ctx, keys) {
   }
   name ??= 'Error';
 
-  if ('cause' in err &&
+  if (ObjectPrototypeHasOwnProperty(err, 'cause') &&
       (keys.length === 0 || !ArrayPrototypeIncludes(keys, 'cause'))) {
     ArrayPrototypePush(keys, 'cause');
   }
@@ -1711,7 +1711,7 @@ function formatError(err, constructor, tag, ctx, keys) {
   // Print errors aggregated into AggregateError
   try {
     const errors = err.errors;
-    if (ArrayIsArray(errors) &&
+    if (ArrayIsArray(errors) && ObjectPrototypeHasOwnProperty(err, 'errors') &&
       (keys.length === 0 || !ArrayPrototypeIncludes(keys, 'errors'))) {
       ArrayPrototypePush(keys, 'errors');
     }
@@ -2292,9 +2292,9 @@ function formatProperty(ctx, value, recurseTimes, key, type, desc,
     if (ctx.getters && (ctx.getters === true ||
           (ctx.getters === 'get' && desc.set === undefined) ||
           (ctx.getters === 'set' && desc.set !== undefined))) {
+      ctx.indentationLvl += 2;
       try {
         const tmp = FunctionPrototypeCall(desc.get, original);
-        ctx.indentationLvl += 2;
         if (tmp === null) {
           str = `${s(`[${label}:`, sp)} ${s('null', 'null')}${s(']', sp)}`;
         } else if (typeof tmp === 'object') {
@@ -2303,11 +2303,11 @@ function formatProperty(ctx, value, recurseTimes, key, type, desc,
           const primitive = formatPrimitive(s, tmp, ctx);
           str = `${s(`[${label}:`, sp)} ${primitive}${s(']', sp)}`;
         }
-        ctx.indentationLvl -= 2;
       } catch (err) {
-        const message = `<Inspection threw (${err.message})>`;
+        const message = `<Inspection threw (${formatValue(ctx, err, recurseTimes)})>`;
         str = `${s(`[${label}:`, sp)} ${message}${s(']', sp)}`;
       }
+      ctx.indentationLvl -= 2;
     } else {
       str = ctx.stylize(`[${label}]`, sp);
     }
