@@ -24,9 +24,8 @@
 const {
   ArrayPrototypePush,
   Float64Array,
-  NumberParseInt,
   ObjectDefineProperties,
-  StringPrototypeEndsWith,
+  ObjectFreeze,
   StringPrototypeSlice,
   SymbolToPrimitive,
 } = primordials;
@@ -41,6 +40,7 @@ const {
   },
   hideStackFrames,
 } = require('internal/errors');
+const { getCIDR } = require('internal/util');
 const { validateInt32 } = require('internal/validators');
 
 const {
@@ -184,9 +184,9 @@ function tmpdir() {
            process.env.TMP ||
            (process.env.SystemRoot || process.env.windir) + '\\temp';
 
-    if (path.length > 1 && StringPrototypeEndsWith(path, '\\') &&
-        !StringPrototypeEndsWith(path, ':\\'))
+    if (path.length > 1 && path[path.length - 1] === '\\' && path[path.length - 2] !== ':') {
       return StringPrototypeSlice(path, 0, -1);
+    }
 
     return path;
   }
@@ -203,60 +203,6 @@ function endianness() {
 }
 endianness[SymbolToPrimitive] = () => kEndianness;
 
-// Returns the number of ones in the binary representation of the decimal
-// number.
-function countBinaryOnes(n) {
-  // Count the number of bits set in parallel, which is faster than looping
-  n = n - ((n >>> 1) & 0x55555555);
-  n = (n & 0x33333333) + ((n >>> 2) & 0x33333333);
-  return ((n + (n >>> 4) & 0xF0F0F0F) * 0x1010101) >>> 24;
-}
-
-function getCIDR(address, netmask, family) {
-  let ones = 0;
-  let split = '.';
-  let range = 10;
-  let groupLength = 8;
-  let hasZeros = false;
-  let lastPos = 0;
-
-  if (family === 'IPv6') {
-    split = ':';
-    range = 16;
-    groupLength = 16;
-  }
-
-  for (let i = 0; i < netmask.length; i++) {
-    if (netmask[i] !== split) {
-      if (i + 1 < netmask.length) {
-        continue;
-      }
-      i++;
-    }
-    const part = StringPrototypeSlice(netmask, lastPos, i);
-    lastPos = i + 1;
-    if (part !== '') {
-      if (hasZeros) {
-        if (part !== '0') {
-          return null;
-        }
-      } else {
-        const binary = NumberParseInt(part, range);
-        const binaryOnes = countBinaryOnes(binary);
-        ones += binaryOnes;
-        if (binaryOnes !== groupLength) {
-          if ((binary & 1) !== 0) {
-            return null;
-          }
-          hasZeros = true;
-        }
-      }
-    }
-  }
-
-  return `${address}/${ones}`;
-}
-
 /**
  * @returns {Record<string, Array<{
  *  address: string,
@@ -299,8 +245,8 @@ function networkInterfaces() {
 }
 
 /**
- * @param {number} [pid=0]
- * @param {number} priority
+ * @param {number} [pid]
+ * @param {number} [priority]
  * @returns {void}
  */
 function setPriority(pid, priority) {
@@ -319,7 +265,7 @@ function setPriority(pid, priority) {
 }
 
 /**
- * @param {number} [pid=0]
+ * @param {number} [pid]
  * @returns {number}
  */
 function getPriority(pid) {
@@ -338,9 +284,9 @@ function getPriority(pid) {
 }
 
 /**
- * @param {{ encoding?: string }} [options=utf8] If `encoding` is set to
- * `'buffer'`, the `username`, `shell`, and `homedir` values will
- * be `Buffer` instances.
+ * @param {{ encoding?: string }} [options] If `encoding` is set to
+ *   `'buffer'`, the `username`, `shell`, and `homedir` values will
+ *   be `Buffer` instances.
  * @returns {{
  *   uid: number,
  *   gid: number,
@@ -359,11 +305,6 @@ function userInfo(options) {
   if (user === undefined)
     throw new ERR_SYSTEM_ERROR(ctx);
 
-  if (isWindows) {
-    user.uid |= 0;
-    user.gid |= 0;
-  }
-
   return user;
 }
 
@@ -390,6 +331,8 @@ module.exports = {
   machine: getMachine,
 };
 
+ObjectFreeze(constants.signals);
+
 ObjectDefineProperties(module.exports, {
   constants: {
     __proto__: null,
