@@ -1,6 +1,7 @@
 #include "node_modules.h"
 #include <cstdio>
 #include "base_object-inl.h"
+#include "compile_cache.h"
 #include "node_errors.h"
 #include "node_external_reference.h"
 #include "node_url.h"
@@ -21,14 +22,20 @@ namespace modules {
 
 using v8::Array;
 using v8::Context;
+using v8::External;
 using v8::FunctionCallbackInfo;
 using v8::HandleScope;
+using v8::Integer;
 using v8::Isolate;
 using v8::Local;
+using v8::LocalVector;
+using v8::Name;
 using v8::NewStringType;
+using v8::Null;
 using v8::Object;
 using v8::ObjectTemplate;
 using v8::Primitive;
+using v8::PropertyCallbackInfo;
 using v8::String;
 using v8::Undefined;
 using v8::Value;
@@ -91,20 +98,32 @@ const BindingData::PackageConfig* BindingData::GetPackageJSON(
 
   auto cache_entry = binding_data->package_configs_.find(path.data());
   if (cache_entry != binding_data->package_configs_.end()) {
-    return &cache_entry->second;
+    auto& cache_value = cache_entry->second;
+    if (cache_value) {
+      return &*cache_value;
+    }
+
+    // If we have a cache entry without a value, we've already
+    // attempted to open and read this path and couldn't (it most
+    // likely doesn't exist)
+    return nullptr;
   }
 
   PackageConfig package_config{};
   package_config.file_path = path;
   // No need to exclude BOM since simdjson will skip it.
   if (ReadFileSync(&package_config.raw_json, path.data()) < 0) {
+    // Add `nullopt` to the package config cache so that we don't
+    // need to open and attempt to read this path again
+    binding_data->package_configs_.insert({std::string(path), std::nullopt});
     return nullptr;
   }
 
   simdjson::ondemand::document document;
   simdjson::ondemand::object main_object;
   simdjson::error_code error =
-      binding_data->json_parser.iterate(package_config.raw_json).get(document);
+      binding_data->json_parser.iterate(simdjson::pad(package_config.raw_json))
+          .get(document);
 
   const auto throw_invalid_package_config = [error_context, path, realm]() {
     if (error_context == nullptr) {
@@ -232,7 +251,7 @@ const BindingData::PackageConfig* BindingData::GetPackageJSON(
   auto cached = binding_data->package_configs_.insert(
       {std::string(path), std::move(package_config)});
 
-  return &cached.first->second;
+  return &*cached.first->second;
 }
 
 void BindingData::ReadPackageJSON(const FunctionCallbackInfo<Value>& args) {
@@ -289,22 +308,24 @@ const BindingData::PackageConfig* BindingData::TraverseParent(
 
     // Stop the search when the process doesn't have permissions
     // to walk upwards
-    if (is_permissions_enabled &&
-        !env->permission()->is_granted(
-            env,
-            permission::PermissionScope::kFileSystemRead,
-            current_path.generic_string())) [[unlikely]] {
-      return nullptr;
+    if (is_permissions_enabled) {
+      if (!env->permission()->is_granted(
+              env,
+              permission::PermissionScope::kFileSystemRead,
+              ConvertGenericPathToUTF8(current_path))) [[unlikely]] {
+        return nullptr;
+      }
     }
 
     // Check if the path ends with `/node_modules`
-    if (current_path.generic_string().ends_with("/node_modules")) {
+    if (current_path.filename() == "node_modules") {
       return nullptr;
     }
 
     auto package_json_path = current_path / "package.json";
+
     auto package_json =
-        GetPackageJSON(realm, package_json_path.string(), nullptr);
+        GetPackageJSON(realm, ConvertPathToUTF8(package_json_path), nullptr);
     if (package_json != nullptr) {
       return package_json;
     }
@@ -313,6 +334,23 @@ const BindingData::PackageConfig* BindingData::TraverseParent(
   return nullptr;
 }
 
+const std::filesystem::path BindingData::NormalizePath(
+    Realm* realm, BufferValue* path_value) {
+  // Check if the path has a trailing slash. If so, add it after
+  // ToNamespacedPath() as it will be deleted by ToNamespacedPath()
+  bool slashCheck = path_value->ToStringView().ends_with(kPathSeparator);
+
+  ToNamespacedPath(realm->env(), path_value);
+
+  auto path = path_value->ToPath();
+
+  if (slashCheck) {
+    path /= "";
+  }
+
+  return path;
+}
+
 void BindingData::GetNearestParentPackageJSON(
     const v8::FunctionCallbackInfo<v8::Value>& args) {
   CHECK_GE(args.Length(), 1);
@@ -320,19 +358,10 @@ void BindingData::GetNearestParentPackageJSON(
 
   Realm* realm = Realm::GetCurrent(args);
   BufferValue path_value(realm->isolate(), args[0]);
-  // Check if the path has a trailing slash. If so, add it after
-  // ToNamespacedPath() as it will be deleted by ToNamespacedPath()
-  bool slashCheck = path_value.ToStringView().ends_with(kPathSeparator);
 
-  ToNamespacedPath(realm->env(), &path_value);
+  auto path = NormalizePath(realm, &path_value);
 
-  std::string path_value_str = path_value.ToString();
-  if (slashCheck) {
-    path_value_str.push_back(kPathSeparator);
-  }
-
-  auto package_json =
-      TraverseParent(realm, std::filesystem::path(path_value_str));
+  auto package_json = TraverseParent(realm, path);
 
   if (package_json != nullptr) {
     args.GetReturnValue().Set(package_json->Serialize(realm));
@@ -346,29 +375,22 @@ void BindingData::GetNearestParentPackageJSONType(
 
   Realm* realm = Realm::GetCurrent(args);
   BufferValue path_value(realm->isolate(), args[0]);
-  // Check if the path has a trailing slash. If so, add it after
-  // ToNamespacedPath() as it will be deleted by ToNamespacedPath()
-  bool slashCheck = path_value.ToStringView().ends_with(kPathSeparator);
 
-  ToNamespacedPath(realm->env(), &path_value);
+  auto path = NormalizePath(realm, &path_value);
 
-  std::string path_value_str = path_value.ToString();
-  if (slashCheck) {
-    path_value_str.push_back(kPathSeparator);
-  }
-
-  auto package_json =
-      TraverseParent(realm, std::filesystem::path(path_value_str));
+  auto package_json = TraverseParent(realm, path);
 
   if (package_json == nullptr) {
     return;
   }
 
-  Local<Value> value =
-      ToV8Value(realm->context(), package_json->type).ToLocalChecked();
-  args.GetReturnValue().Set(value);
+  Local<Value> value;
+  if (ToV8Value(realm->context(), package_json->type).ToLocal(&value)) {
+    args.GetReturnValue().Set(value);
+  }
 }
 
+template <bool return_only_type>
 void BindingData::GetPackageScopeConfig(
     const FunctionCallbackInfo<Value>& args) {
   CHECK_GE(args.Length(), 1);
@@ -400,11 +422,22 @@ void BindingData::GetPackageScopeConfig(
     }
 
     auto file_url = url::FileURLToPath(realm->env(), *package_json_url);
-    CHECK(file_url);
+    if (!file_url) {
+      url::ThrowInvalidURL(realm->env(), resolved.ToStringView(), std::nullopt);
+      return;
+    }
     error_context.specifier = resolved.ToString();
     auto package_json = GetPackageJSON(realm, *file_url, &error_context);
     if (package_json != nullptr) {
-      return args.GetReturnValue().Set(package_json->Serialize(realm));
+      if constexpr (return_only_type) {
+        Local<Value> value;
+        if (ToV8Value(realm->context(), package_json->type).ToLocal(&value)) {
+          args.GetReturnValue().Set(value);
+        }
+        return;
+      } else {
+        return args.GetReturnValue().Set(package_json->Serialize(realm));
+      }
     }
 
     auto last_href = std::string(package_json_url->get_href());
@@ -422,15 +455,20 @@ void BindingData::GetPackageScopeConfig(
     }
   }
 
+  if constexpr (return_only_type) {
+    return;
+  }
+
+  // If the package.json could not be found return a string containing a path
+  // to the non-existent package.json file in the initial requested location
   auto package_json_url_as_path =
       url::FileURLToPath(realm->env(), *package_json_url);
   CHECK(package_json_url_as_path);
-  return args.GetReturnValue().Set(
-      String::NewFromUtf8(realm->isolate(),
-                          package_json_url_as_path->c_str(),
-                          NewStringType::kNormal,
-                          package_json_url_as_path->size())
-          .ToLocalChecked());
+  Local<Value> ret;
+  if (ToV8Value(realm->context(), *package_json_url_as_path, realm->isolate())
+          .ToLocal(&ret)) {
+    args.GetReturnValue().Set(ret);
+  }
 }
 
 void FlushCompileCache(const FunctionCallbackInfo<Value>& args) {
@@ -460,13 +498,21 @@ void EnableCompileCache(const FunctionCallbackInfo<Value>& args) {
     THROW_ERR_INVALID_ARG_TYPE(env, "cacheDir should be a string");
     return;
   }
+
+  EnableOption option = EnableOption::DEFAULT;
+  if (args.Length() > 1 && args[1]->IsTrue()) {
+    option = EnableOption::PORTABLE;
+  }
+
   Utf8Value value(isolate, args[0]);
-  CompileCacheEnableResult result = env->EnableCompileCache(*value);
-  std::vector<Local<Value>> values = {
-      v8::Integer::New(isolate, static_cast<uint8_t>(result.status)),
-      ToV8Value(context, result.message).ToLocalChecked(),
-      ToV8Value(context, result.cache_directory).ToLocalChecked()};
-  args.GetReturnValue().Set(Array::New(isolate, values.data(), values.size()));
+  CompileCacheEnableResult result = env->EnableCompileCache(*value, option);
+  Local<Value> values[3];
+  values[0] = v8::Integer::New(isolate, static_cast<uint8_t>(result.status));
+  if (ToV8Value(context, result.message).ToLocal(&values[1]) &&
+      ToV8Value(context, result.cache_directory).ToLocal(&values[2])) {
+    args.GetReturnValue().Set(
+        Array::New(isolate, &values[0], arraysize(values)));
+  }
 }
 
 void GetCompileCacheDir(const FunctionCallbackInfo<Value>& args) {
@@ -477,9 +523,150 @@ void GetCompileCacheDir(const FunctionCallbackInfo<Value>& args) {
     args.GetReturnValue().Set(v8::String::Empty(isolate));
     return;
   }
-  args.GetReturnValue().Set(
-      ToV8Value(context, env->compile_cache_handler()->cache_dir())
-          .ToLocalChecked());
+
+  Local<Value> ret;
+  if (ToV8Value(context, env->compile_cache_handler()->cache_dir())
+          .ToLocal(&ret)) {
+    args.GetReturnValue().Set(ret);
+  }
+}
+
+void GetCompileCacheEntry(const FunctionCallbackInfo<Value>& args) {
+  Isolate* isolate = args.GetIsolate();
+  CHECK(args[0]->IsString());  // TODO(joyeecheung): accept buffer.
+  CHECK(args[1]->IsString());
+  CHECK(args[2]->IsUint32());
+  Local<Context> context = isolate->GetCurrentContext();
+  Environment* env = Environment::GetCurrent(context);
+  if (!env->use_compile_cache()) {
+    return;
+  }
+  Local<String> source = args[0].As<String>();
+  Local<String> filename = args[1].As<String>();
+  CachedCodeType type =
+      static_cast<CachedCodeType>(args[2].As<v8::Uint32>()->Value());
+  auto* cache_entry =
+      env->compile_cache_handler()->GetOrInsert(source, filename, type);
+  if (cache_entry == nullptr) {
+    return;
+  }
+
+  v8::LocalVector<v8::Name> names(isolate,
+                                  {FIXED_ONE_BYTE_STRING(isolate, "external")});
+  v8::LocalVector<v8::Value> values(isolate,
+                                    {v8::External::New(isolate, cache_entry)});
+  if (cache_entry->cache != nullptr) {
+    Debug(env,
+          DebugCategory::COMPILE_CACHE,
+          "[compile cache] retrieving transpile cache for %s %s...",
+          cache_entry->type_name(),
+          cache_entry->source_filename);
+
+    std::string_view cache(
+        reinterpret_cast<const char*>(cache_entry->cache->data),
+        cache_entry->cache->length);
+    Local<Value> transpiled;
+    // TODO(joyeecheung): convert with simdutf and into external strings
+    if (!ToV8Value(context, cache).ToLocal(&transpiled)) {
+      Debug(env, DebugCategory::COMPILE_CACHE, "failed\n");
+      return;
+    } else {
+      Debug(env, DebugCategory::COMPILE_CACHE, "success\n");
+    }
+    names.push_back(FIXED_ONE_BYTE_STRING(isolate, "transpiled"));
+    values.push_back(transpiled);
+  } else {
+    Debug(env,
+          DebugCategory::COMPILE_CACHE,
+          "[compile cache] no transpile cache for %s %s\n",
+          cache_entry->type_name(),
+          cache_entry->source_filename);
+  }
+  args.GetReturnValue().Set(Object::New(
+      isolate, v8::Null(isolate), names.data(), values.data(), names.size()));
+}
+
+static void PathHelpersLazyGetter(Local<v8::Name> name,
+                                  const PropertyCallbackInfo<Value>& info) {
+  Isolate* isolate = info.GetIsolate();
+  // This getter has no JavaScript function representation and is not
+  // invoked in the creation context.
+  // When this getter is invoked in a vm context, the `Realm::GetCurrent(info)`
+  // returns a nullptr and retrieve the creation context via `this` object and
+  // get the creation Realm.
+  Local<Value> receiver_val = info.This();
+  if (!receiver_val->IsObject()) {
+    THROW_ERR_INVALID_INVOCATION(isolate);
+    return;
+  }
+  Local<Object> receiver = receiver_val.As<Object>();
+  Local<Context> context;
+  if (!receiver->GetCreationContext().ToLocal(&context)) {
+    THROW_ERR_INVALID_INVOCATION(isolate);
+    return;
+  }
+  Environment* env = Environment::GetCurrent(context);
+
+  node::Utf8Value url(isolate, info.Data());
+  auto file_url = ada::parse(url.ToStringView());
+  CHECK(file_url);
+  auto file_path = url::FileURLToPath(env, *file_url);
+  CHECK(file_path.has_value());
+  std::string_view ret_view = file_path.value();
+
+  node::Utf8Value utf8name(isolate, name);
+  auto plain_name = utf8name.ToStringView();
+  if (plain_name == "dirname") {
+#ifdef _WIN32
+#define PATH_SEPARATOR '\\'
+#else
+#define PATH_SEPARATOR '/'
+#endif
+    auto index = ret_view.rfind(PATH_SEPARATOR);
+    CHECK(index != std::string_view::npos);
+    ret_view.remove_suffix(ret_view.size() - index);
+#undef PATH_SEPARATOR
+  }
+  Local<Value> ret;
+  if (!ToV8Value(context, ret_view, isolate).ToLocal(&ret)) {
+    return;
+  }
+  info.GetReturnValue().Set(ret);
+}
+void InitImportMetaPathHelpers(const FunctionCallbackInfo<Value>& args) {
+  // target, url, shouldSetDirnameAndFilename, resolve
+  CHECK_GE(args.Length(), 2);
+  CHECK(args[0]->IsObject());
+  CHECK(args[1]->IsString());
+
+  Isolate* isolate = args.GetIsolate();
+  Local<Context> context = isolate->GetCurrentContext();
+  Environment* env = Environment::GetCurrent(context);
+
+  auto target = args[0].As<Object>();
+
+  // N.B.: Order is important to keep keys in alphabetical order.
+  if (target
+          ->SetLazyDataProperty(
+              context, env->dirname_string(), PathHelpersLazyGetter, args[1])
+          .IsNothing() ||
+      target
+          ->SetLazyDataProperty(
+              context, env->filename_string(), PathHelpersLazyGetter, args[1])
+          .IsNothing())
+    return;
+}
+void SaveCompileCacheEntry(const FunctionCallbackInfo<Value>& args) {
+  Isolate* isolate = args.GetIsolate();
+  Local<Context> context = isolate->GetCurrentContext();
+  Environment* env = Environment::GetCurrent(context);
+  DCHECK(env->use_compile_cache());
+  CHECK(args[0]->IsExternal());
+  CHECK(args[1]->IsString());  // TODO(joyeecheung): accept buffer.
+  auto* cache_entry =
+      static_cast<CompileCacheEntry*>(args[0].As<External>()->Value());
+  Utf8Value utf8(isolate, args[1].As<String>());
+  env->compile_cache_handler()->MaybeSave(cache_entry, utf8.ToStringView());
 }
 
 void BindingData::CreatePerIsolateProperties(IsolateData* isolate_data,
@@ -494,10 +681,15 @@ void BindingData::CreatePerIsolateProperties(IsolateData* isolate_data,
             target,
             "getNearestParentPackageJSON",
             GetNearestParentPackageJSON);
-  SetMethod(isolate, target, "getPackageScopeConfig", GetPackageScopeConfig);
+  SetMethod(
+      isolate, target, "getPackageScopeConfig", GetPackageScopeConfig<false>);
+  SetMethod(isolate, target, "getPackageType", GetPackageScopeConfig<true>);
   SetMethod(isolate, target, "enableCompileCache", EnableCompileCache);
   SetMethod(isolate, target, "getCompileCacheDir", GetCompileCacheDir);
   SetMethod(isolate, target, "flushCompileCache", FlushCompileCache);
+  SetMethod(isolate, target, "getCompileCacheEntry", GetCompileCacheEntry);
+  SetMethod(isolate, target, "saveCompileCacheEntry", SaveCompileCacheEntry);
+  SetMethod(isolate, target, "setLazyPathHelpers", InitImportMetaPathHelpers);
 }
 
 void BindingData::CreatePerContextProperties(Local<Object> target,
@@ -507,19 +699,38 @@ void BindingData::CreatePerContextProperties(Local<Object> target,
   Realm* realm = Realm::GetCurrent(context);
   realm->AddBindingData<BindingData>(target);
 
-  std::vector<Local<Value>> compile_cache_status_values;
   Isolate* isolate = context->GetIsolate();
+  LocalVector<Value> compile_cache_status_values(isolate);
 
 #define V(status)                                                              \
   compile_cache_status_values.push_back(                                       \
       FIXED_ONE_BYTE_STRING(isolate, #status));
   COMPILE_CACHE_STATUS(V)
+#undef V
 
   USE(target->Set(context,
                   FIXED_ONE_BYTE_STRING(isolate, "compileCacheStatus"),
                   Array::New(isolate,
                              compile_cache_status_values.data(),
                              compile_cache_status_values.size())));
+
+  LocalVector<Name> cached_code_type_keys(isolate);
+  LocalVector<Value> cached_code_type_values(isolate);
+
+#define V(type, value)                                                         \
+  cached_code_type_keys.push_back(FIXED_ONE_BYTE_STRING(isolate, #type));      \
+  cached_code_type_values.push_back(Integer::New(isolate, value));             \
+  DCHECK_EQ(value, cached_code_type_values.size() - 1);
+  CACHED_CODE_TYPES(V)
+#undef V
+
+  USE(target->Set(context,
+                  FIXED_ONE_BYTE_STRING(isolate, "cachedCodeTypes"),
+                  Object::New(isolate,
+                              Null(isolate),
+                              cached_code_type_keys.data(),
+                              cached_code_type_values.data(),
+                              cached_code_type_keys.size())));
 }
 
 void BindingData::RegisterExternalReferences(
@@ -527,10 +738,14 @@ void BindingData::RegisterExternalReferences(
   registry->Register(ReadPackageJSON);
   registry->Register(GetNearestParentPackageJSONType);
   registry->Register(GetNearestParentPackageJSON);
-  registry->Register(GetPackageScopeConfig);
+  registry->Register(GetPackageScopeConfig<false>);
+  registry->Register(GetPackageScopeConfig<true>);
   registry->Register(EnableCompileCache);
   registry->Register(GetCompileCacheDir);
   registry->Register(FlushCompileCache);
+  registry->Register(GetCompileCacheEntry);
+  registry->Register(SaveCompileCacheEntry);
+  registry->Register(InitImportMetaPathHelpers);
 }
 
 }  // namespace modules
