@@ -116,19 +116,23 @@ class EnvironmentOptions : public Options {
   bool abort_on_uncaught_exception = false;
   std::vector<std::string> conditions;
   bool detect_module = true;
+  bool disable_sigusr1 = false;
   bool print_required_tla = false;
   bool require_module = true;
   std::string dns_result_order;
   bool enable_source_maps = false;
+  bool experimental_addon_modules = false;
   bool experimental_eventsource = false;
   bool experimental_fetch = true;
   bool experimental_websocket = true;
   bool experimental_sqlite = true;
   bool experimental_webstorage = false;
+#ifdef NODE_OPENSSL_HAS_QUIC
+  bool experimental_quic = false;
+#endif
   std::string localstorage_file;
   bool experimental_global_navigator = true;
   bool experimental_global_web_crypto = true;
-  bool experimental_wasm_modules = false;
   bool experimental_import_meta_resolve = false;
   std::string input_type;  // Value of --input-type
   bool entry_is_url = false;
@@ -136,12 +140,13 @@ class EnvironmentOptions : public Options {
   std::vector<std::string> allow_fs_read;
   std::vector<std::string> allow_fs_write;
   bool allow_addons = false;
+  bool allow_inspector = false;
   bool allow_child_process = false;
   bool allow_wasi = false;
   bool allow_worker_threads = false;
   bool experimental_repl_await = true;
   bool experimental_vm_modules = false;
-  bool async_context_frame = false;
+  bool async_context_frame = true;
   bool expose_internals = false;
   bool force_node_api_uncaught_exceptions_policy = false;
   bool frozen_intrinsics = false;
@@ -168,6 +173,8 @@ class EnvironmentOptions : public Options {
   std::string cpu_prof_name;
   bool cpu_prof = false;
   bool experimental_network_inspection = false;
+  bool experimental_worker_inspection = false;
+  bool experimental_inspector_network_resource = false;
   std::string heap_prof_dir;
   std::string heap_prof_name;
   static const uint64_t kDefaultHeapProfInterval = 512 * 1024;
@@ -176,8 +183,8 @@ class EnvironmentOptions : public Options {
 #endif  // HAVE_INSPECTOR
   std::string redirect_warnings;
   std::string diagnostic_dir;
-  std::string env_file;
-  std::string optional_env_file;
+  std::vector<std::string> env_file;
+  std::vector<std::string> optional_env_file;
   bool has_env_file_string = false;
   bool test_runner = false;
   uint64_t test_runner_concurrency = 0;
@@ -191,7 +198,9 @@ class EnvironmentOptions : public Options {
   bool test_runner_update_snapshots = false;
   std::vector<std::string> test_name_pattern;
   std::vector<std::string> test_reporter;
+  std::string test_rerun_failures_path;
   std::vector<std::string> test_reporter_destination;
+  std::string test_global_setup_path;
   bool test_only = false;
   bool test_udp_no_try_send = false;
   std::string test_isolation = "process";
@@ -224,6 +233,7 @@ class EnvironmentOptions : public Options {
   bool watch_mode = false;
   bool watch_mode_report_to_parent = false;
   bool watch_mode_preserve_output = false;
+  std::string watch_mode_kill_signal = "SIGTERM";
   std::vector<std::string> watch_mode_paths;
 
   bool syntax_check_only = false;
@@ -233,6 +243,7 @@ class EnvironmentOptions : public Options {
   bool force_repl = false;
 
   bool insecure_http_parser = false;
+  bool use_env_proxy = false;
 
   bool tls_min_v1_0 = false;
   bool tls_min_v1_1 = false;
@@ -246,13 +257,15 @@ class EnvironmentOptions : public Options {
 
   std::vector<std::string> preload_esm_modules;
 
-  bool experimental_strip_types = false;
+  bool strip_types = HAVE_AMARO;
   bool experimental_transform_types = false;
 
   std::vector<std::string> user_argv;
 
   bool report_exclude_env = false;
   bool report_exclude_network = false;
+  std::string experimental_config_file_path;
+  bool experimental_default_config_file = false;
 
   inline DebugOptions* get_debug_options() { return &debug_options_; }
   inline const DebugOptions& debug_options() const { return debug_options_; }
@@ -274,6 +287,8 @@ class PerIsolateOptions : public Options {
   bool report_uncaught_exception = false;
   bool report_on_signal = false;
   bool experimental_shadow_realm = false;
+  std::string max_old_space_size_percentage;
+  std::string max_old_space_size;
   int64_t stack_trace_limit = 10;
   std::string report_signal = "SIGUSR2";
   bool build_snapshot = false;
@@ -281,6 +296,8 @@ class PerIsolateOptions : public Options {
   inline EnvironmentOptions* get_per_env_options();
   void CheckOptions(std::vector<std::string>* errors,
                     std::vector<std::string>* argv) override;
+  void HandleMaxOldSpaceSizePercentage(std::vector<std::string>* errors,
+                                       std::string* max_old_space_size);
 
   inline std::shared_ptr<PerIsolateOptions> Clone() const;
 
@@ -338,6 +355,7 @@ class PerProcessOptions : public Options {
   bool ssl_openssl_cert_store = false;
 #endif
   bool use_openssl_ca = false;
+  bool use_system_ca = false;
   bool use_bundled_ca = false;
   bool enable_fips_crypto = false;
   bool force_fips_crypto = false;
@@ -370,7 +388,7 @@ class PerProcessOptions : public Options {
 namespace options_parser {
 
 HostPort SplitHostPort(const std::string& arg,
-    std::vector<std::string>* errors);
+                       std::vector<std::string>* errors);
 void GetOptions(const v8::FunctionCallbackInfo<v8::Value>& args);
 std::string GetBashCompletion();
 
@@ -384,6 +402,45 @@ enum OptionType {
   kHostPort,
   kStringList,
 };
+std::unordered_map<std::string, OptionType> MapEnvOptionsFlagInputType();
+std::unordered_map<std::string, OptionType> MapOptionsByNamespace(
+    std::string namespace_name);
+std::unordered_map<std::string,
+                   std::unordered_map<std::string, options_parser::OptionType>>
+MapNamespaceOptionsAssociations();
+std::vector<std::string> MapAvailableNamespaces();
+
+// Define all namespace entries
+#define OPTION_NAMESPACE_LIST(V)                                               \
+  V(kNoNamespace, "")                                                          \
+  V(kTestRunnerNamespace, "testRunner")                                        \
+  V(kWatchNamespace, "watch")
+
+enum class OptionNamespaces {
+#define V(name, _) name,
+  OPTION_NAMESPACE_LIST(V)
+#undef V
+};
+
+inline const std::string NamespaceEnumToString(OptionNamespaces ns) {
+  switch (ns) {
+#define V(name, string_value)                                                  \
+  case OptionNamespaces::name:                                                 \
+    return string_value;
+    OPTION_NAMESPACE_LIST(V)
+#undef V
+    default:
+      return "";
+  }
+}
+
+inline constexpr auto AllNamespaces() {
+  return std::array{
+#define V(name, _) OptionNamespaces::name,
+      OPTION_NAMESPACE_LIST(V)
+#undef V
+  };
+}
 
 template <typename Options>
 class OptionsParser {
@@ -398,39 +455,59 @@ class OptionsParser {
   // These methods add a single option to the parser. Optionally, it can be
   // specified whether the option should be allowed from environment variable
   // sources (i.e. NODE_OPTIONS).
-  void AddOption(const char* name,
-                 const char* help_text,
-                 bool Options::*field,
-                 OptionEnvvarSettings env_setting = kDisallowedInEnvvar,
-                 bool default_is_true = false);
-  void AddOption(const char* name,
-                 const char* help_text,
-                 uint64_t Options::*field,
-                 OptionEnvvarSettings env_setting = kDisallowedInEnvvar);
-  void AddOption(const char* name,
-                 const char* help_text,
-                 int64_t Options::*field,
-                 OptionEnvvarSettings env_setting = kDisallowedInEnvvar);
-  void AddOption(const char* name,
-                 const char* help_text,
-                 std::string Options::*field,
-                 OptionEnvvarSettings env_setting = kDisallowedInEnvvar);
-  void AddOption(const char* name,
-                 const char* help_text,
-                 std::vector<std::string> Options::*field,
-                 OptionEnvvarSettings env_setting = kDisallowedInEnvvar);
-  void AddOption(const char* name,
-                 const char* help_text,
-                 HostPort Options::*field,
-                 OptionEnvvarSettings env_setting = kDisallowedInEnvvar);
-  void AddOption(const char* name,
-                 const char* help_text,
-                 NoOp no_op_tag,
-                 OptionEnvvarSettings env_setting = kDisallowedInEnvvar);
-  void AddOption(const char* name,
-                 const char* help_text,
-                 V8Option v8_option_tag,
-                 OptionEnvvarSettings env_setting = kDisallowedInEnvvar);
+
+  // default_is_true is only a hint in printing help text, it does not
+  // affect the default value of the option. Set the default value in the
+  // Options struct instead.
+  void AddOption(
+      const char* name,
+      const char* help_text,
+      bool Options::*field,
+      OptionEnvvarSettings env_setting = kDisallowedInEnvvar,
+      bool default_is_true = false,
+      OptionNamespaces namespace_id = OptionNamespaces::kNoNamespace);
+  void AddOption(
+      const char* name,
+      const char* help_text,
+      uint64_t Options::*field,
+      OptionEnvvarSettings env_setting = kDisallowedInEnvvar,
+      OptionNamespaces namespace_id = OptionNamespaces::kNoNamespace);
+  void AddOption(
+      const char* name,
+      const char* help_text,
+      int64_t Options::*field,
+      OptionEnvvarSettings env_setting = kDisallowedInEnvvar,
+      OptionNamespaces namespace_id = OptionNamespaces::kNoNamespace);
+  void AddOption(
+      const char* name,
+      const char* help_text,
+      std::string Options::*field,
+      OptionEnvvarSettings env_setting = kDisallowedInEnvvar,
+      OptionNamespaces namespace_id = OptionNamespaces::kNoNamespace);
+  void AddOption(
+      const char* name,
+      const char* help_text,
+      std::vector<std::string> Options::*field,
+      OptionEnvvarSettings env_setting = kDisallowedInEnvvar,
+      OptionNamespaces namespace_id = OptionNamespaces::kNoNamespace);
+  void AddOption(
+      const char* name,
+      const char* help_text,
+      HostPort Options::*field,
+      OptionEnvvarSettings env_setting = kDisallowedInEnvvar,
+      OptionNamespaces namespace_id = OptionNamespaces::kNoNamespace);
+  void AddOption(
+      const char* name,
+      const char* help_text,
+      NoOp no_op_tag,
+      OptionEnvvarSettings env_setting = kDisallowedInEnvvar,
+      OptionNamespaces namespace_id = OptionNamespaces::kNoNamespace);
+  void AddOption(
+      const char* name,
+      const char* help_text,
+      V8Option v8_option_tag,
+      OptionEnvvarSettings env_setting = kDisallowedInEnvvar,
+      OptionNamespaces namespace_id = OptionNamespaces::kNoNamespace);
 
   // Adds aliases. An alias can be of the form "--option-a" -> "--option-b",
   // or have a more complex group expansion, like
@@ -520,12 +597,15 @@ class OptionsParser {
   // - A type.
   // - A way to store/access the property value.
   // - The information of whether it may occur in an env var or not.
+  // - A default value (if applicable).
+  // - A namespace ID (optional) to allow for namespacing of options.
   struct OptionInfo {
     OptionType type;
     std::shared_ptr<BaseOptionField> field;
     OptionEnvvarSettings env_setting;
     std::string help_text;
     bool default_is_true = false;
+    std::string namespace_id;
   };
 
   // An implied option is composed of the information on where to store a
@@ -564,6 +644,15 @@ class OptionsParser {
   friend void GetCLIOptionsInfo(
       const v8::FunctionCallbackInfo<v8::Value>& args);
   friend std::string GetBashCompletion();
+  friend std::unordered_map<std::string, OptionType>
+  MapEnvOptionsFlagInputType();
+  friend std::unordered_map<std::string, OptionType> MapOptionsByNamespace(
+      std::string namespace_name);
+  friend std::vector<std::string> MapAvailableNamespaces();
+  friend void GetEnvOptionsInputType(
+      const v8::FunctionCallbackInfo<v8::Value>& args);
+  friend void GetOptionsAsFlags(
+      const v8::FunctionCallbackInfo<v8::Value>& args);
 };
 
 using StringVector = std::vector<std::string>;
