@@ -2,13 +2,14 @@
 const common = require('../common');
 const assert = require('assert');
 const dc = require('diagnostics_channel');
+const { pathToFileURL } = require('url');
 
 const trace = dc.tracingChannel('module.import');
 const events = [];
 let lastEvent;
 
 function track(name) {
-  return (event) => {
+  return common.mustCall((event) => {
     // Verify every event after the first is the same object
     if (events.length) {
       assert.strictEqual(event, lastEvent);
@@ -16,23 +17,20 @@ function track(name) {
     lastEvent = event;
 
     events.push({ name, ...event });
-  };
+  });
 }
 
 trace.subscribe({
-  start: common.mustCall(track('start')),
-  end: common.mustCall(track('end')),
-  asyncStart: common.mustCall(track('asyncStart')),
-  asyncEnd: common.mustCall(track('asyncEnd')),
-  error: common.mustNotCall(track('error')),
+  start: track('start'),
+  end: track('end'),
+  asyncStart: track('asyncStart'),
+  asyncEnd: track('asyncEnd'),
+  error: common.mustNotCall('error'),
 });
 
 import('http').then(
   common.mustCall((result) => {
-    let expectedParentURL = module.filename.replaceAll('\\', '/');
-    expectedParentURL = common.isWindows ?
-      `file:///${expectedParentURL}` :
-      `file://${expectedParentURL}`;
+    const expectedParentURL = pathToFileURL(module.filename).href;
     // Verify order and contents of each event
     assert.deepStrictEqual(events, [
       {
